#! /bin/sh
#
###########################################################################
#    This file is part of kal.
#    Copyright (C) 1999, 2000 Frank W. Josellis
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################
#
# Shell script for the configuration and installation of kal.
#
########################################################################### 
#

SRC_FILES="Makefile kal.c kal.h days.c calendar.c kal2tex.c latex.h \
kal2html.c easter.c moon.c moon.h riset.c hebrew.c islamic.c \
getopt.c getopt1.c getopt.h"

ALL_FILES="$SRC_FILES kal.1.gz kal-sites kal-sitesview COPYING README"

#
# Check if the sources are here
#
check_files() { 
    # We need read permission for the source files
    for i in $ALL_FILES 
    do 
	if [ ! -f $i -o ! -r $i ]; then
	    echo "$0: Can't access file: $i"
 	    failed=1
	fi
    done

    if [ "$failed" != "" ]; then
	exit 1
    fi
}    


#
# Get and test the LaTeX length specifications 
#
get_length() {
    TEX_UNITS="cm mm in pt bp pc dd cc em ex"
    LENGTH="`echo $1 | \
	sed -n -e '/^[+-]\{0,1\}[0-9]*[.,]\{0,1\}[0-9]*[a-z]\{1,\}$/p'`"
	
    
    if [ "$LENGTH" != "" ]; then
	VALUE="`echo $LENGTH | sed -n -e 's/[a-z]*$//gp' | \
		sed -e 's/[.,]$//g'`" 
	
	ABSVALUE="`echo $VALUE | sed -e 's/^[+-]//g'`"
	UNIT="`echo $LENGTH | sed -e 's/[+-.,0-9]//g'`"
	
	for i in $TEX_UNITS; do
	    if [ "$UNIT" = "$i" ]; then
		MATCH=1
	    fi
	done

	if [ "$MATCH" != "" -a "$ABSVALUE" != "" ]; then
	    LENGTH=$VALUE$UNIT
	    return 0
	else
	    echo "This looks like a strange TeX length specification: $LENGTH"
	    echo "Assuming default value instead."
	    echo "Press <Return> to continue ..."
	    read 
	    return 1
	    	
	fi
    else
	echo "This looks like an illegal TeX length specification to me."
	echo "Assuming default value."
	echo "Press <Return> to continue ..."
	read
	return 1
    fi
}

#
# Get the directory names
#
get_pathname() {
    
    # First get rid of leading "./" or "../"
    # These would cause the sub-makes to install to wrong destinations.

    dir=`echo $1 | sed -n -e 's/^\.\///p'`
    if [ "$dir" != "" ]; then
	dir=$PWD/$dir
    else
	dir=`echo $1 | sed -n -e 's/^\.\.\///p'`
	if [ "$dir" != "" ]; then
	    dir=`dirname $PWD`/$dir 
	fi
    fi
    
    if [ "$dir" = "" ]; then
	dir=$1
    fi
    
    # Attach $(prefix) to relative pathnames
    dir="`echo $dir | sed -n -e '/^\//p'`"
    if [ "$dir" = "" ]; then
	dir='$(prefix)'/$1
    fi
}


# Don't waste time with configuration if the sources are not here
check_files

# Get the version from kal.h
#VERSION="`cat kal.h | sed -n -e '/VERSION/s/^#define[ ]*VERSION[ ]*\"//gp' | \
#sed -e 's/\"$//'`" 
#sed -e 's/^version[ ]*//g' | sed -e 's/\"$//'`" 

# Get the current settings from Makefile
prefix="`sed -n -e '/^prefix/s/^prefix[ ]*=[ ]*//gp' Makefile`"
bindir="`sed -n -e '/^bindir/s/^bindir[ ]*=[ ]*//gp' Makefile`"
mandir="`sed -n -e '/^mandir/s/^mandir[ ]*=[ ]*//gp' Makefile`"
man1dir="`sed -n -e '/^man1dir/s/^man1dir[ ]*=[ ]*//gp' Makefile`"
cat1dir="`sed -n -e '/^cat1dir/s/^cat1dir[ ]*=[ ]*//gp' Makefile`"
docdir="`sed -n -e '/^docdir/s/^docdir[ ]*=[ ]*//gp' Makefile`"
kaldoc="`sed -n -e '/^kaldoc/s/^kaldoc[ ]*=[ ]*//gp' Makefile`"
libdir="`sed -n -e '/^libdir/s/^libdir[ ]*=[ ]*//gp' Makefile`"
kallib="`sed -n -e '/^kallib/s/^kallib[ ]*=[ ]*//gp' Makefile`"
VERSION="`sed -n -e '/^VERSION/s/^VERSION[ ]*=[ ]*//gp' Makefile`"
FLAGS="`sed -n -e '/^DEFINES/s/^DEFINES[ ]*=[ ]*//gp' Makefile`"

# Get current flags
if [ "`echo $FLAGS | sed -n -e '/-D_EUROPEAN_STYLE/p'`" != "" ]; then
    USE_AMERICAN=n
else
    USE_AMERICAN=y
fi

if [ "`echo $FLAGS | sed -n -e '/-D_ANGLICAN/p'`" != "" ]; then
    USE_GREGORIAN=n
else
    USE_GREGORIAN=y
fi

if [ "`echo $FLAGS | sed -n -e '/-D_NO_COLORS/p'`" != "" ]; then
    USE_COLORS=n
else
    USE_COLORS=y
fi

FILTER_ID="`echo $FLAGS | sed -n -e '/-D_FILTER=/p' | \
	sed -e 's/.*-D_FILTER=//g' | sed -e 's/[ ].*$//g'`"
case $FILTER_ID in
    0)  USE_FILTER="none";;
    1)  USE_FILTER="more";;
    2)  USE_FILTER="less";;
    3)  USE_FILTER="most";;
esac
if [ "$USE_FILTER" = "" ]; then
    FILTER_ID=0 ; USE_FILTER="none"
fi	


# Get the current settings from kal.h
SITESDIR="`cat kal.h | \
sed -n -e '/^#define[ ]*SITES_SYS/s/^#define[ ]*SITES_SYS[ ]*\"//gp' | \
sed -e 's/\"$//'`"
SITESDIR="`dirname $SITESDIR`"

# Get the current settings from latex.h
A4TOP="`cat latex.h | \
sed -n -e '/^#define[ ]*A4TOP/s/^#define[ ]*A4TOP[ ]*\"//gp' | \
sed -e 's/\"$//'`"
A4ODD="`cat latex.h | \
sed -n -e '/^#define[ ]*A4ODD/s/^#define[ ]*A4ODD[ ]*\"//gp' | \
sed -e 's/"$//'`"
LETTERTOP="`cat latex.h | \
sed -n -e '/^#define[ ]*LETTERTOP/s/^#define[ ]*LETTERTOP[ ]*\"//gp' | \
sed -e 's/\"$//'`"
LETTERODD="`cat latex.h | \
sed -n -e '/^#define[ ]*LETTERODD/s/^#define[ ]*LETTERODD[ ]*\"//gp' | \
sed -e 's/\"$//'`"

if [ "`echo $FLAGS | sed -n -e '/-D_A4PAPER/p'`" = "" ]; then
    USE_LETTER=y
    PAPERSIZE=letterpaper
    TOPMARGIN=$LETTERTOP
    ODDSIDEMARGIN=$LETTERODD
else
    USE_LETTER=n
    PAPERSIZE=a4paper
    TOPMARGIN=$A4TOP
    ODDSIDEMARGIN=$A4ODD
fi

#
# Welcome page
#
clear
echo -e "
This is the 'kal' installation routine.
\nkal is free software and you are welcome to distribute copies of it
under certain conditions. There is ABSOLUTELY NO WARRANTY for kal.
For full details, see the file COPYING (shipped along with this program),
or write to:
\tFree Software Foundation, Inc.
\t675 Mass Ave\n\tCambridge, MA 02139\n\tUSA"

echo -e "
The installation script provides an interface for the following steps:
\t(1) Configure the program for your needs. 
\t(2) Build the kal binary.
\t(3) Install the files at their chosen destinations.

You will be prompted for some input. If this is a Yes-No choice [Y|n], 
enter 'y' or 'n', or hit <Return> to accept the upper case alternative. 
Otherwise you should enter some text or press <Return> to confirm the 
default which then is displayed in square brackets. Hit control-C to quit 
the installation process at any time."

echo "
Begin with configuration [Y|n] " 
read input

#
# Begin of configuration section. Get the DEFINES for compilation.
#
if [ "$input" != "n" ]; then
    if [ "$USE_AMERICAN" = "y" ]; then
	YESNO="Y|n"
    else
	YESNO="y|N"
    fi
    echo -e "
Calendar Style
  kal offers two different styles for the calendar layout: 
  the \"American\" style (weeks begin on Sunday) and the 
  \"European\" style (weeks begin on Monday). Weeks and  
  week numbers of the \"European\" style calendar conform 
  with the ISO-8601 standard. The default setting can be 
  overridden by the layout option \`-l'.
    
Accept \"American\" style [$YESNO] "  
    read input
    case $input in
	[yY]*)  USE_AMERICAN=y;;
	[nN]*)  USE_AMERICAN=n;;
    esac        	    
    

    if [ "$USE_GREGORIAN" = "y" ]; then
	YESNO="Y|n"
    else
	YESNO="y|N"
    fi
    echo -e "
\nGregorian Reformation
  kal can assume the Gregorian Reformation to have occurred 
  in October 1582 or in September 1752. This refers to the 
  dates when it was first established and when it was accepted 
  in England and the Dominion, respectively. This default can
  be overridden on the command line by the \`-g' option. 
    
Accept Gregorian Reformation in 1582 [$YESNO] "  
    read input
    case $input in
	[yY]*)  USE_GREGORIAN=y;;
	[nN]*)  USE_GREGORIAN=n;;
    esac        	    


    if [ "$USE_COLORS" = "y" ]; then
	YESNO="Y|n"
    else
	YESNO="y|N"
    fi
    echo -e "
\nANSI Colors
  The kal terminal output uses ANSI colors to improve the 
  representation. If the default use of colors is enabled,
  try the command line option \`-c' for monochrome output. 
  Conversely, entering 'n' here means that \`-c' switches
  to the colored mode.  
    
Accept ANSI colors as default [$YESNO] "  
    read input
    case $input in
	[yY]*)  USE_COLORS=y;;
	[nN]*)  USE_COLORS=n;;
    esac        	    


    echo -e "
\nDefault Filter 
  If kal is invoked to print calendars of a year or to show
  the help screen, the output would normally not fit into a 
  typical terminal, e.g. the console display. However, if a
  default filter \`more',\`less', or \`most' is chosen here, 
  kal will in such case address the output to that filter. 
  Specifying \`none' causes kal to write everything to stdout 
  by default. The default can be overridden by the \`-f' option. 
  Note that ANSI colors are not interpreted by \`most'.  
    
Choose one of \`none', \`more', \`less', or \`most' [$USE_FILTER] "  
    read input
    ID=-1
    case $input in
	"none")  ID=0;;
	"more")  ID=1;;
	"less")  ID=2;;
	"most")  ID=3;;
    esac
    if [ $ID -gt -1 ]; then
	FILTER_ID=$ID
	USE_FILTER="$input"
    fi    

    

    if [ "$USE_LETTER" = "y" ]; then
	YESNO="Y|n"
    else
	YESNO="y|N"
    fi
    echo -e "
\nLaTeX Output
  If you consider to print LaTeX formatted calendars with
  kal you will probably have to adjust the margin settings 
  to your printer equipment. kal comes with pre-configured 
  defaults for \"letterpaper\" and \"a4paper\", but these may 
  give non-centered output on many printers. So, first make a 
  choice for your preferred format, \"letterpaper\" or \"a4paper\". 
  
Accept paper format \"letterpaper\" [$YESNO] "  
    read input
    case $input in
    [nN]*)  
	USE_LETTER=n
	PAPERSIZE=a4paper 
	TOPMARGIN=$A4TOP
	ODDSIDEMARGIN=$A4ODD;;
    [yY]*)
	USE_LETTER=y
	PAPERSIZE=letterpaper
	TOPMARGIN=$LETTERTOP
	ODDSIDEMARGIN=$LETTERODD;;
    esac

#
# Get TOPMARGIN and ODDSIDEMARGIN for latex.h
#
    echo "
See and/or edit the margin values for LaTeX output [y|N] "
    read input
    case $input in
	[yY]*) input="y";;
    esac	
	
    if [ "$input" = "y" ]; then
	echo -e "
The chosen paper format is \"$PAPERSIZE\". kal suggests:
  TOPMARGIN\t$TOPMARGIN
  ODDSIDEMARGIN\t$ODDSIDEMARGIN
    
Enter a TOPMARGIN length or confirm with <Return> [$TOPMARGIN] "
	read input
	if [ "$input" != "" ]; then
	    get_length $input
	    if [ $? -eq 0 ]; then 
		TOPMARGIN=$LENGTH
	    fi
	fi
	echo \
"Enter an ODDSIDEMARGIN length or confirm with <Return> [$ODDSIDEMARGIN] "
	read input
	if [ "$input" != "" ]; then
	    get_length $input
	    if [ $? -eq 0 ]; then
		ODDSIDEMARGIN=$LENGTH
	    fi
	fi

	if [ "$PAPERSIZE" = "a4paper" ]; then
	    A4TOP=$TOPMARGIN
	    A4ODD=$ODDSIDEMARGIN
	else
	    LETTERTOP=$TOPMARGIN
	    LETTERODD=$ODDSIDEMARGIN
	fi
    fi


#
# Get the installation locations
# 
    echo -e "\n
Directories
  It remains to determine the destination directories for the executable,
  the man page, the doc files, and the sites file. A typical installation 
  would require a directory structure of the form

    TOPLEVEL/BIN
    TOPLEVEL/MAN/man1
    TOPLEVEL/DOC/kal-$VERSION
    TOPLEVEL/LIB/kal-$VERSION

  Thus you will be requested first to specify TOPLEVEL and then BIN,
  MAN, DOC, and LIB relative to the TOPLEVEL directory. For instance, 
  if you plan to build a binary package suitable for installation with
  your preferred software administration tool, you merely have to set 
  the TOPLEVEL and may leave the rest unchanged. However, if you enter 
  an absolute pathname for BIN, MAN, DOC, or LIB the TOPLEVEL will be 
  ignored in that case. 
  
TOPLEVEL directory [$prefix] "
    read input
    if [ "$input" != "" ]; then
	input="`echo $input | sed -e 's/[/]*~[+-]*//g' | sed -e 's/\/*$//g'`"
	get_pathname $input
	if [ "`echo $dir | sed -n -e '/$(prefix)/p'`" != "" ]; then
	    prefix=$prefix/`echo $dir | sed -e 's/$(prefix)\///'`
	else
	    prefix=$dir
	fi	    
    fi

    stripdir="`echo $bindir | sed -e 's/\$([a-z]*)\///g'`" 
    echo "BIN directory (relative to \`$prefix') [$stripdir] "
    read input
    if [ "$input" != "" ]; then
	input="`echo $input | sed -e 's/[/]*~[+-]*//g' | sed -e 's/\/*$//g'`"
	get_pathname $input
    else
	get_pathname $stripdir 
    fi
    bindir=$dir
    
    stripdir="`echo $mandir | sed -e 's/\$([a-z]*)\///g'`" 
    echo "MAN directory (relative to \`$prefix') [$stripdir] "
    read input
    if [ "$input" != "" ]; then
	input="`echo $input | sed -e 's/[/]*~[+-]*//g' | sed -e 's/\/*$//g'`"
	get_pathname $input
    else
	get_pathname $stripdir
    fi
    mandir=$dir
    man1dir='$(mandir)'/man1
    cat1dir='$(mandir)'/cat1

    stripdir="`echo $docdir | sed -e 's/\$([a-z]*)\///g'`"
    echo "DOC directory (relative to \`$prefix') [$stripdir] "
    read input
    if [ "$input" != "" ]; then
	input="`echo $input | sed -e 's/[/]*~[+-]*//g' | sed -e 's/\/*$//g'`"
	get_pathname $input
    else
	get_pathname $stripdir
    fi
    docdir=$dir
    kaldoc='$(docdir)'/kal-'$(VERSION)'

    stripdir="`echo $libdir | sed -e 's/\$([a-z]*)\///g'`"
    echo "LIB directory (relative to \`$prefix') [$stripdir] "
    read input
    if [ "$input" != "" ]; then
	input="`echo $input | sed -e 's/[/]*~[+-]*//g' | sed -e 's/\/*$//g'`"
	get_pathname $input
    else
	get_pathname $stripdir
    fi
    libdir=$dir    
    kallib='$(libdir)'/kal-'$(VERSION)'

    # Try to get SITESDIR from libdir
    if [ "`echo $libdir | sed -n -e '/$(prefix)/p'`" != "" ]; then
	LIBDIR="$prefix/`echo $libdir | sed -e 's/$(prefix)\///'`"
    else
	LIBDIR=$libdir
    fi
    KALLIB="$LIBDIR/kal-$VERSION"
    SITESDIR=`echo $KALLIB | sed -e 's/^.*\/usr/\/usr/'`

    echo -e "
The sites file
  By default kal reads the geographic data required for computation 
  of rise and set times from the sites file kal-sites which is going
  to be installed in 
    
    KALLIB = $KALLIB
  
  In order to tell kal where to look for the sites file, its absolute 
  pathname has to be specified. At the current state the corresponding 
  entry in kal.h will point to

    SITESDIR = $SITESDIR

  Note: If you plan a straightforward installation of kal right now, you 
  should have KALLIB = SITESDIR. If, however, you intend to build a binary 
  package of kal in some subdirectory ROOT for later installation, then 
  you should have KALLIB = ROOT/SITESDIR. If this is not yet correctly set,
  make sure that SITESDIR points to the final destination of the sites file.
  
SITESDIR [$SITESDIR] "
    read input
    if [ "$input" != "" ]; then
	SITESDIR=$input
    fi

    #
    # Collect the DEFINES in one string 
    #
    FLAGS=
    if [ "$USE_AMERICAN" = "n" ]; then
        FLAGS="$FLAGS-D_EUROPEAN_STYLE "
    fi

    if [ "$USE_GREGORIAN" = "n" ]; then
        FLAGS="$FLAGS-D_ANGLICAN "
    fi

    if [ "$USE_COLORS" = "n" ]; then
        FLAGS="$FLAGS-D_NO_COLORS "
    fi

    if [ $FILTER_ID -ne 0 ]; then
        FLAGS="$FLAGS-D_FILTER=$FILTER_ID "
    fi
    
    if [ "$USE_LETTER" = "n" ]; then
        FLAGS="$FLAGS-D_A4PAPER "
    fi
    echo -e "\nDone with configuration."
fi
 
   
#
# Show current settings 
#
echo -e "\nCurrent flags:"
if [ "$USE_AMERICAN" = "y" ]; then
    echo -e "\tCalendar style is American"
else	
    echo -e "\tCalendar style is European"
fi

if [ "$USE_GREGORIAN" = "y" ]; then
    echo -e "\tGregorian Reformation is assumed in October 1582" 
else
    echo -e "\tGregorian Reformation is assumed in September 1752" 
fi

if [ "$USE_LETTER" = "y" ]; then
    echo -e "\tLaTeX output uses \"letterpaper\""
else
    echo -e "\tLaTeX output uses \"a4paper\""
fi

if [ "$USE_COLORS" = "y" ]; then
    echo -e "\tUse colors by default: Yes"
else
    echo -e "\tUse colors by default: No"
fi
    
echo -e "\tUse default filter: $USE_FILTER"


#
# The upper case variables are introduced for control only.
#
if [ "`echo $bindir | sed -n -e '/$(prefix)/p'`" != "" ]; then
    BINDIR="$prefix/`echo $bindir | sed -e 's/$(prefix)\///'`"
else
    BINDIR=$bindir
fi
   
if [ "`echo $mandir | sed -n -e '/$(prefix)/p'`" != "" ]; then
    MANDIR="$prefix/`echo $mandir | sed -e 's/$(prefix)\///'`"
else
    MANDIR=$mandir
fi
  
MAN1DIR="$MANDIR/`echo $man1dir | sed -e 's/$(mandir)\///'`"
CAT1DIR="$MANDIR/`echo $cat1dir | sed -e 's/$(mandir)\///'`"
    
if [ "`echo $docdir | sed -n -e '/$(prefix)/p'`" != "" ]; then
    DOCDIR="$prefix/`echo $docdir | sed -e 's/$(prefix)\///'`"
else
    DOCDIR=$docdir
fi
    
KALDOC="$DOCDIR/kal-$VERSION"
    
if [ "`echo $libdir | sed -n -e '/$(prefix)/p'`" != "" ]; then
    LIBDIR="$prefix/`echo $libdir | sed -e 's/$(prefix)\///'`"
else
    LIBDIR=$libdir
fi

KALLIB="$LIBDIR/kal-$VERSION"


echo -e "\nCurrent file destinations:
\tbin: $BINDIR
\tman: $MAN1DIR 
\tdoc: $KALDOC
\tlib: $KALLIB

Current setting for system sites file:
\t$SITESDIR/kal-sites

Current margin settings for latex.h:
\t\"$PAPERSIZE\": TOPMARGIN $TOPMARGIN, ODDSIDEMARGIN $ODDSIDEMARGIN"



#
# Begin of build section
#
echo -e "\n\nBuild the kal binary [Y|n] "
read input
if [ "$input" != "" ]; then
    case $input in
	[yY]*) input=y;;
    esac
else
    input=y
fi
    
if [ "$input" = "y" ]; then
    # We need all permissions for the current directory 
    if [ ! -r $PWD -o ! -w $PWD ]; then
	echo "Permission denied: $PWD"
	exit 1;
    fi
    
    #
    # Update kal.h
    #
    if [ ! -f kal.h.orig ]; then
	cp -p kal.h kal.h.orig
    fi
    
    cat kal.h.orig |
    sed -e '/#define[ ]*VERSION/c\
#define VERSION "'$VERSION'"
' | sed -e '/#define[ ]*SITES_SYS/c\
#define SITES_SYS "'$SITESDIR'/kal-sites"
' > kal.h

    # Don't keep an identical copy of kal..h
    if [ "`diff kal.h.orig kal.h`" = "" ]; then
	mv kal.h.orig kal.h
    fi


    #
    # Update latex.h
    #
    if [ ! -f latex.h.orig ]; then
	cp -p latex.h latex.h.orig
    fi

    cat latex.h.orig |
    sed -e '/#define[ ]*LETTERTOP/c\
#define LETTERTOP     "'$LETTERTOP'"
' | sed -e '/#define[ ]*LETTERODD/c\
#define LETTERODD     "'$LETTERODD'"
' | sed -e '/#define[ ]*A4TOP/c\
#define A4TOP         "'$A4TOP'"
' | sed -e '/#define[ ]*A4ODD/c\
#define A4ODD         "'$A4ODD'"
' > latex.h
    
    # Don't keep an identical copy of latex.h
    if [ "`diff latex.h.orig latex.h`" = "" ]; then
	mv latex.h.orig latex.h
    fi


    #
    # Check for gcc
    #
    which gcc > /dev/null
    if [ $? -eq 0 ]; then
	CC=gcc
    else
	CC=cc
    fi
    
    #
    # Update Makefile
    #
    if [ ! -f Makefile.orig ]; then
	cp -p Makefile Makefile.orig
    fi

    cat Makefile.orig |
    sed -e '/^prefix/c\
prefix   = '$prefix'
' | sed -e '/^bindir/c\
bindir   = '$bindir'
' | sed -e '/^mandir/c\
mandir   = '$mandir'
' | sed -e '/^man1dir/c\
man1dir  = '$man1dir'
' | sed -e '/^cat1dir/c\
cat1dir  = '$cat1dir'
' | sed -e '/^docdir/c\
docdir   = '$docdir'
' | sed -e '/^kaldoc/c\
kaldoc   = '$kaldoc'
' | sed -e '/^libdir/c\
libdir   = '$libdir'
' | sed -e '/^kallib/c\
kallib   = '$kallib'
' | sed -e '/^DEFINES/c\
DEFINES  = '"$FLAGS"'
' | sed -e '/^VERSION/c\
VERSION  = '$VERSION'
' | sed -e '/^CC/c\
CC       = '$CC'
' > Makefile


    #
    # make
    # 
    make realclean ; make
    if [ $? -eq 0 ]; then
	echo -e "\nDone with compilation."
    fi
fi


#
# Begin of installation section
#
if [ -x kal ]; then
    echo -e "
Install everything [Y|n] "
    read input
    if [ "$input" != "" ]; then
	case $input in
	    [yY]*) input=y;;
	esac
    else
	input=y
    fi
    
    if [ "$input" = "y" ]; then
	make install
	if [ $? -eq 0 ]; then
	    echo -e "\nDone with installation."
	fi
    fi
fi
