/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* easter.c */

#include <stdio.h>
#include <stdlib.h>
#include "kal.h"

extern int extra;
extern char *monthname[];

/* Table of the Paschal full moon for the Julian calendar: */ 
static int FM[19] = { 5,25,13, 2,22,10,30,18, 7,27,15, 4,24,12, 1,21, 9,29,17};

/*-------------------------------------------------------------------------*/
/* Function `gregorian_epact'                                              */
/* The Epact for the Gregorian calendar                                    */
/*-------------------------------------------------------------------------*/
int gregorian_epact(year)
     int year;
{
  int epact, c;
  
  /*
   * The Easter algorithm is based on the 19-year Metonic cycle.
   * The first step is determine the Epact of a given year, which 
   * is approximately the age of moon at the beginning of that year. 
   * For the Julian calendar one has 
   * 
   *    Epact(y) = (11 * y%19)%30 + 8
   * 
   * The value y%19 + 1 is the so-called the "Golden Number" of the 
   * year y.
   * 
   * For the Gregorian calendar this value has to be corrected. Firstly,  
   * subtract the number of Julian leap days that do not exist in the
   * Gregorian calendar. Secondly, take into account that the 19-year
   * Metonic cycle does is not exactly an integer multiple of lunations. 
   * In Oudin's Easter formula this is done by adding
   * (c - (c - 17)//25)//3 with c = y//100 (integer division). Changing 
   * this to 
   *
   *   (c + 1 - (c + 8)//25)//3
   * 
   * which yields the same result for c > 14, one finds that 8 days
   * are added in 2500 years. (The formula adds 1/3 day every century
   * except for c = 17 + 25 * n. Thus one may replace it by the simpler
   * expression (8 * c + 13)//25 that yields the same sequence of values
   * for all nonnegative c.)
   *   
   * Finally, the result mod 30 gives the the Gregorian Epact.
   *
   */

  c = year/100; 
  epact = (11 * (year%19))%30 + 8 - (3 * c + 3)/4 + (8 * c + 13)/25;
  epact += 90;   /* to avoid negative values */ 
  return(epact%30);
}

/*-------------------------------------------------------------------------*/
/* Function `paschal_full_moon'                                            */
/* The "official" full moon after the vernal equinox                       */
/*-------------------------------------------------------------------------*/
int paschal_full_moon(year)
     int year;
{
  int fm, epact = gregorian_epact(year);

  fm = (42 - epact)%30 + 1;
  if(fm > 19) fm++;
  if(fm == 19) fm--;
  if(epact == 25 && year%19 > 10) fm--;
  
  return(fm);
}


/*-------------------------------------------------------------------------*/
/* Function `easter_sunday'                                                */
/* Computes the date of Easter Sunday (1-3000)                             */
/*-------------------------------------------------------------------------*/
void easter_sunday(year,color,out)
     int year, color; 
     FILE *out;
{
  int d, day, month;
  
  if((extra && year < 1753) || (!extra && year < 1583))   
    day = FM[year%19];
  else
    day = paschal_full_moon(year);
  
  month = day > 20 ? 3 : 4;
  d = daynum(month,day,year);
  d = (d + 8 - (d - 1)%7 - daynum(1,1,year));
  
  month = 1;
  while(d > mlength(month,year)){
    d -= mlength(month,year);
    month++;
  }
  
  fprintf(out,"Easter Sunday %i: ",year);
  fprintf(out,"%s%s %i%s\n",MODE_G1(color),monthname[month - 1],d,
	  MODE_N(color));
}

/*-------------------------------------------------------------------------*/
/* Function `easter_sunday_greek'                                          */
/* Computes the date of Greek Orthodox Easter Sunday (1-3000)              */
/*-------------------------------------------------------------------------*/
void easter_sunday_greek(year,color,out)
     int year, color; 
     FILE *out;
{
  int d, day, month, centuries = 0, periods = 0;

  day = FM[year%19];
  month = day > 20 ? 3 : 4;

  if(year > 1600){
    centuries  = (year - 1601)/100; 
    periods    = (year - 1601)/400; 
  }
  d = daynum(month,day,year) + centuries - periods;
  d -= (extra && year > 1700 && year < 1753) ? 1 : 0;
  d += ((extra && year > 1752) || (!extra && year > 1582)) ? 10 : 0;
  if(year > 1600 + 100*extra && !(year%100) && (year/100)%4) d++;
  d = (d + 8 - (d - 1)%7 - daynum(1,1,year));

  month = 1;
  while(d > mlength(month,year)){
    d -= mlength(month,year);
    month++;
  }
  fprintf(out,"Greek Orthodox Easter Sunday %i: ",year);
  fprintf(out,"%s%s %i%s\n",MODE_G1(color),monthname[month - 1],d,
	  MODE_N(color));
}
