/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* hebrew.c */

#include <stdio.h>
#include "kal.h"
#include "moon.h"


extern char *hebrew_monthname_0[], *hebrew_monthname_1[], *monthname[];


/*-------------------------------------------------------------------------*/
/* Function `hebrew_year_months'                                           */
/* The number of months of a Hebrew year                                   */
/*-------------------------------------------------------------------------*/
int hebrew_year_months(year)
     int year;
{
  /* 
   * The leap years are the years 3, 6, 8, 11, 14, 17, and 19 in a 
   * nineteen-year cycle beginning with the Hebrew year 1.
   * A (Metonic) nineteen-year cycle consists of 235 lunations. Each 
   * year in the cycle is assigned a number 12 or 13 of lunations. 
   * This can be done by setting 
   * 
   * lun(y) := entier((235y + 1)/19), y = 0,...,19.
   * 
   * Indeed, then 
   * 
   * months(y) := lun(y) - lun(y - 1), y = 1,...,19, 
   * 
   * yields the desired number of months of the year y. On the 
   * other hand, months(y) = 13 exactly if (235y + 1)%19 < 7. Since
   * 235 = 7 mod 19, the Hebrew leap years are characterized by:
   */
  
  return(((7 * year + 1)%19 < 7) ? 13 : 12);
}

/*-------------------------------------------------------------------------*/
/* Function `hebrew_delay_1'                                               */
/*-------------------------------------------------------------------------*/
unsigned int hebrew_delay_1(year)
     int year;
{
  unsigned int lunations, days, halakim;

  /* Lunation constants:
   * The mean lunation is set to 29 days, 12 hours, 793 halakim, which 
   * is about the length of a synodic month. By definition one has 
   * 1 helek = 10/3 sec, i.e., 1 hour = 1080 halakim. Hence, expressed 
   * in days and halakim, the mean lunation is 29 days, 13753 halakim. 
   *
   * We note that 1 day = 25920 halakim.
   *
   * The day Tishri 1 'year', the beginning of a Hebrew year, is 
   * derived from the day of Tishri molad, the "birth" of the moon, 
   * and the "dehiyyot", 4 rules of postponement. Tishri molad of 
   * the Hebrew year 1 occurred on day 2 at 5 hours, 204 halakim 
   * (Jerusalem local time). Consequently, Tishri molad of the year 1 
   * causes a shift by 1 day, 5604 halakim.
   *
   * A Hebrew day begins at 6 PM. It is convenient to introduce an 
   * additional shift by 6 hours = 6480 halakim, thus translating the 
   * the initial point backwards to noon. The total initial shift is 
   * therefore 1 day, 12084 halakim.
   */
  
  lunations = (235 * (year - 1) + 1)/19; 
  halakim   = 12084 + (13753 * lunations);
  days      = 1 + (lunations * 29) + halakim/25920;
  
  /*
   * The value of 'days' is the number of days elapsed when Tishri 
   * molad of 'year' occurs. 
   *
   * Now the first dehiyyah comes in which states that Tishri 1 is to 
   * be postponed by one day if Tishri molad falls on day 1, 4, or 6. 
   * (The days of the week are numbered rather than named. Only day 7, 
   * Sabbath, is given a specific name.) Thus Tishri 1 is delayed by 
   * one day if days = n mod 7 with n = 0, 3, or 5. This is equivalent
   * to 3 * days = n mod 7 with n = 0, 1, or 2.  
   * 
   * The second dehiyyah requires to shift Tishri 1 by one day if Tishri 
   * molad occurs at or after 18 hours, i.e., after noon. This rule is
   * satisfied in view of the extra shift by 6 hours.
   *
   * Thus one gets an approximate value for Tishri 1 'year':
   */
  
  if((3 * days)%7 < 3) days++;
  return(days);
}

/*-------------------------------------------------------------------------*/
/* Function `hebrew_delay_2'                                               */
/*-------------------------------------------------------------------------*/
int hebrew_delay_2(year)
     int year;
{
  int last, this, next;

  /*
   * There are two more dehiyyot which prevent ordinary years from
   * exceeding 355 days and avoid leap years shorter than 383 days.
   * These correct the values for Tishri 1 returned by hebrew_delay_1():
   *
   * If the pre-computed values for Tishri 1 of 'year' and 'year + 1' 
   * differ by 356 days, Tishri 1 'year' is to be postponed by two days.
   *
   * If the pre-computed values for Tishri 1 of 'year - 1' and 'year'
   * differ by 382 days, Tishri 1 'year' is to be postponed by one day. 
   */
  
  last = hebrew_delay_1(year - 1);
  this = hebrew_delay_1(year);
  next = hebrew_delay_1(year + 1);
  
  return(((next - this) == 356) ? 2 : (((this - last) == 382) ? 1 : 0));
}

/*-------------------------------------------------------------------------*/
/* Function `hebrew_year_days'                                             */
/* Compute the length of a Hebrew year.                                    */
/*-------------------------------------------------------------------------*/
int hebrew_year_days(year)
     int year;
{
  /*
   * By hebrew_delay_1 + hebrew_delay_2 we get the day of Tishri 1.
   * The length of 'year' is the number of days between Tishri 1 of 
   * 'year' and Tishri 1 of 'year + 1'.
   */
   
  return(hebrew_delay_1(year + 1) + hebrew_delay_2(year + 1) 
	 - hebrew_delay_1(year) - hebrew_delay_2(year));
}  

/*-------------------------------------------------------------------------*/
/* Function `hebrew_month_days'                                            */
/*-------------------------------------------------------------------------*/
int hebrew_month_days(month,year)
     int month, year;
{
  int d;

  switch(hebrew_year_months(year)){
  case 13:
    switch(month){
    case 4:
    case 7:
    case 9:
    case 11:
    case 13:
      d = 29;
      break;
    default:
      d = 30;  
    }
    break;
  default:
    switch(month){
    case 4:
    case 6:
    case 8:
    case 10:
    case 12:
      d = 29;
      break;
    default:
      d = 30;  
    }
  }
  
  /* 
   * The length of months 2 (Heshvan) and 3 (Kislev) depends
   * on the length of the year.
   */

  if((month == 2 && hebrew_year_days(year)%10 != 5) ||
     (month == 3 && hebrew_year_days(year)%10 == 3))
    d = 29;    

  return(d);
}


/*-------------------------------------------------------------------------*/
/* Function `hebrew2jd'                                                    */
/*-------------------------------------------------------------------------*/
double hebrew2jd(month,day,year)
     int month, day, year;
{
  int m;
  double jd;

  jd = HEBREW_EPOCH + hebrew_delay_1(year) + hebrew_delay_2(year) + day; 
  
  if(month > 1)
    for(m = 1; m < month; m++)
      jd += hebrew_month_days(m,year);
  
  return(jd);
}

/*-------------------------------------------------------------------------*/
/* Function `jd2hebrew'                                                    */
/*-------------------------------------------------------------------------*/
void jd2hebrew(jd,month,day,year)
     double jd; 
     int *month, *day, *year;
{
  register unsigned int N, j;
 
  N = (unsigned int) (floor(jd) + 0.5 - HEBREW_EPOCH);

  *year = 1;
  while(N > (unsigned int) (hebrew2jd(1,1,*year + 1) - HEBREW_EPOCH)) 
    (*year)++;
  N -= (unsigned int) (hebrew2jd(1,1,*year) - HEBREW_EPOCH);

  *month = 1;
  while(N > (j = hebrew_month_days(*month,*year))){
    (*month)++;
    N -= j;
  }
    
  *day = N;
}
  
/*-------------------------------------------------------------------------*/
/* Function `hebrew_fmt_days_between'                                      */
/* Format for days detween dates                                           */
/*-------------------------------------------------------------------------*/
void hebrew_fmt_days_between(m1,d1,y1,m2,d2,y2,color,out)
     int m1, d1, y1, m2, d2, y2, color;
     FILE *out;
{
  register int m = m2, y = y2, months, days, years;

  if(d2 < d1){
    days = d2 + hebrew_month_days(m1,y1) - d1;
    m--;
    if(!m){
      y--;
      m = hebrew_year_months(y);
    }
  }
  else
    days = d2 - d1;

  if(hebrew_year_months(y) == 13 && hebrew_year_months(y1) == 12)
    m1 += (m1 > 5) ? 1 : 0;
  else
    if(hebrew_year_months(y) == 12 && hebrew_year_months(y1) == 13)
      m1 += (m1 > 6) ? -1 : 0;

  if(m < m1){
    y--;
    if(hebrew_year_months(y + 1) == 13 && hebrew_year_months(y) == 12)
      m1 += (m1 > 6) ? -1 : 0;
    else
      if(hebrew_year_months(y + 1) == 12 && hebrew_year_months(y) == 13)
	m1 += (m1 > 5) ? 1 : 0;
    months = m - m1 + hebrew_year_months(y);
  }
  else
    months = m - m1;
  
  years = y - y1;

  fprintf(out,"%s%i%s year%s, %s%i%s month%s, %s%i%s day%s\n",
	  MODE_G1(color),EPL2(years,MODE_N(color)),
	  MODE_G1(color),EPL2(months,MODE_N(color)),
	  MODE_G1(color),EPL2(days,MODE_N(color)));
}


/*-------------------------------------------------------------------------*/
/* Function `hebrew_day_of_week'                                           */
/*-------------------------------------------------------------------------*/
int hebrew_day_of_week(month,day,year)
     int month, day, year;
{
  unsigned int N;
  
  N = (unsigned int) (hebrew2jd(month,day,year) - HEBREW_EPOCH) - 1;
  return(N%7);
} 

/*-------------------------------------------------------------------------*/
/* Function `hebrew_monthkal'                                              */
/* Formats the calendar for a Hebrew month                                 */
/*-------------------------------------------------------------------------*/
void hebrew_monthkal(month,day,year,color,out)
     int month, day, year, color; 
     FILE *out;
{
  int i, j, N, d, a[6][7];
  char *name;
  
  
  /* Format the head */
  i = 1;
  j = year;
  while(j /= 10) i++;

  name = (hebrew_year_months(year) > 12) ? 
    hebrew_monthname_1[month - 1] : hebrew_monthname_0[month - 1];
    
  N = 19 - strlen(name) - i;
  N = N%2 ? (N+1)/2 : N/2;
  fprintf(out,"%s%s %i\n", void_string(N), name, year);
  fprintf(out,"D1 D2 D3 D4 D5 D6  S\n");

  /* Format the month matrix */
  N = 1;
  d = hebrew_day_of_week(month,1,year);
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      a[i][j] = N - d;
      N++;
    }
  }

  N = hebrew_month_days(month,year);
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      if(a[i][j] < 1 || a[i][j] > N) fprintf(out,"   ");
      else
	if(a[i][j] != day) 
	  fprintf(out,"%2i ", a[i][j]);
	else 
	  fprintf(out,"%s%2i%s ",MODE_R1(color),a[i][j],MODE_N(color));
    }
    fprintf(out,"\n");
  }
  fprintf(out,"\n");
}


/*-------------------------------------------------------------------------*/
/* Function `hebrew_yearkal_head'                                          */
/* Header for a Hebrew year's calendar                                     */
/*-------------------------------------------------------------------------*/
void hebrew_yearkal_head(k,year,out)
     int k, year;
     FILE *out;
{
  int l, N;
  char *name[3];

  if(k < 4){
    for(l = 0; l < 3; l++){
      name[l] = (hebrew_year_months(year) == 12) ? 
	hebrew_monthname_0[3*k + l] : hebrew_monthname_1[3*k + l]; 

      N = 20 - strlen(name[l]);
      N = N%2 ? (N+1)/2 : N/2;
      fprintf(out,"%s%s%s   ",void_string(N),name[l],
	     void_string(20 - N - strlen(name[l]))); 
    }
    fprintf(out,"\n");
    for(l = 0; l < 3; l++)  
      fprintf(out,"D1 D2 D3 D4 D5 D6  S   ");
    fprintf(out,"\n");
  }
  else {
    name[0] = hebrew_monthname_1[12];
    N = 20 - strlen(name[0]);
      N = N%2 ? (N+1)/2 : N/2;
      fprintf(out,"%s%s%s   ",void_string(N),name[0],
	     void_string(20 - N - strlen(name[0]))); 
  
      fprintf(out,"\nD1 D2 D3 D4 D5 D6  S   \n");
  }
}

/*-------------------------------------------------------------------------*/
/* Function `hebrew_yearkal'                                               */
/* Formatted output of a Hebrew year's calendar                            */
/*-------------------------------------------------------------------------*/
void hebrew_yearkal(year,out)
     int year;
     FILE *out;
{
  int i, j, k, l, N, d, a[6][7][3], rows;

  /* Year as title */
  i = 1;
  j = year;
  while(j /= 10) i++;
  N = 66 - i;
  N = N%2 ? (N + 1)/2 : N/2;
  fprintf(out,"%s%i\n\n", void_string(N), year);

  /* Format the block of month matrices with headers */
  rows = (hebrew_year_months(year) == 12) ? 4 : 5;
  for(k = 0; k < rows; k++){
    hebrew_yearkal_head(k,year,out);

    if(k < 4)
      for(l = 0; l < 3; l++){
	N = 1;
	d = hebrew_day_of_week(3*k + 1 + l,1,year);
	for(i = 0; i < 6; i++){
	  for(j = 0; j < 7; j++){
	    a[i][j][l] = N - d;
	    N++;
	  }
	}
      }
    else {
      N = 1;
      d = hebrew_day_of_week(13,1,year);
      for(i = 0; i < 6; i++){
	for(j = 0; j < 7; j++){
	  a[i][j][0] = N - d;
	  N++;
	}
      }
    }

    for(i = 0; i < 6; i++){
      if(k < 4)
	for(l = 0; l < 3; l++){
	  N = hebrew_month_days(3*k + 1 + l,year);
	  for(j = 0; j < 7; j++){
	    if(a[i][j][l] < 1 || a[i][j][l] > N)  fprintf(out,"   ");
	    else fprintf(out,"%2i ", a[i][j][l]);
	  }
	  fprintf(out,"  ");
	}
      else {
	N = hebrew_month_days(13,year);
	for(j = 0; j < 7; j++){
	  if(a[i][j][0] < 1 || a[i][j][0] > N)  fprintf(out,"   ");
	  else fprintf(out,"%2i ", a[i][j][0]);
	}
	fprintf(out,"  ");
      }
      fprintf(out,"\n");
    }
  }
  fprintf(out,"\n");
}


/*-------------------------------------------------------------------------*/
/* Function `RoshHaShanah'                                                 */
/* Computes the day of Rosh Ha-Shanah                                      */
/*-------------------------------------------------------------------------*/
void RoshHaShanah(year,color,out)
     int year, color; 
     FILE *out;
{
  unsigned int month, day;

  inv_julian(hebrew2jd(1,1,year + 3761),&month,&day,&year);
  fprintf(out,"Jewish New Year %i: ",year);
  fprintf(out,"%s%s %i%s",MODE_G1(color),monthname[month - 1],day,
	  MODE_N(color));
  fprintf(out," (Rosh Ha-Shanah %i)\n",year + 3761);
}


/*-------------------------------------------------------------------------*/
/* Function `print_hebrew_day'                                             */
/* Takes a Julian date as input and prints the corresponding Hebrew date   */
/*-------------------------------------------------------------------------*/
void print_hebrew_day(jd,color,out)
     double jd; 
     int color; 
     FILE *out;
{
  int month, day, year;
  
  jd2hebrew(jd,&month,&day,&year);
  fprintf(out,"Hebrew:\t\t");
  fprintf(out,"%s",MODE_G1(color));
  if(hebrew_year_months(year) - 12)
    fprintf(out,"%s",hebrew_monthname_1[month - 1]);
  else
    fprintf(out,"%s",hebrew_monthname_0[month - 1]);
  fprintf(out," %i %i",day,year);
  fprintf(out,"%s",MODE_N(color));
  fprintf(out,"  (%02i/%02i/%i)",month,day,year);
  day = ((unsigned int)(jd - HEBREW_EPOCH) - 1)%7;
  if(day)
    fprintf(out,"  Day %i",day);
  else
    fprintf(out,"  Sabbath");
  fprintf(out,"\n");
}
