/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* islamic.c */

#include <stdio.h>
#include <math.h>
#include "kal.h"


extern char *islamic_monthname[];

/*-------------------------------------------------------------------------*/
/* Function `islamic_year_days'                                            */
/* The number of days of an Islamic year                                   */
/*-------------------------------------------------------------------------*/
int islamic_year_days(year)
     int year;
{
  /* 
   * The leap years are the years 2, 5, 7, 10, 13, 16, 18, 21, 24, 
   * 26, and 29 in thirty-year cycle beginning with the Islamic year 1.
   * A thirty-year cycle consists 10631 days. Each year in the cycle is 
   * assigned a number 354 or 355 of days. This is actally done by 
   * setting 
   * 
   * total(y) := entier((10631y + 14)/30), y = 0,...,30.
   * 
   * Indeed, then 
   * 
   * days(y) := total(y) - total(y - 1), y = 1,...,30, 
   * 
   * yields the desired number of days of the year y. Then one has
   * days(y) = 355 exactly if (10631y + 14)%30 < 11. Since
   * 10631 = 11 mod 30, the Islamic leap years are characterized by:
   */
  
  return(((11 * year + 14)%30 < 11) ? 355 : 354);
}

/*-------------------------------------------------------------------------*/
/* Function `islamic_month_days'                                           */
/* The number of days of an Islamic month                                  */
/*-------------------------------------------------------------------------*/
int islamic_month_days(month,year)
     int month, year;
{
  int d;

  d = (month%2) ? 30 : 29;
  return((month == 12 && islamic_year_days(year) == 355) ? 30 : d);
}

/*-------------------------------------------------------------------------*/
/* Function `islamic2jd'                                                   */
/* The Julian date of an Islamic date                                      */
/*-------------------------------------------------------------------------*/
double islamic2jd(month,day,year)
     int month, day, year;
{
  double jd;

  /* The number of days since ISLAMIC_EPOCH: */
  jd = day + ceil(29.5 * (month - 1)) + (10631 * (year - 1) + 14)/30;

  return(jd + ISLAMIC_EPOCH);
}


/*-------------------------------------------------------------------------*/
/* Function `jd2islamic'                                                   */
/* The Islamic date of a Julian  date                                      */
/*-------------------------------------------------------------------------*/
void jd2islamic(jd,month,day,year)
     double jd; 
     int *month, *day, *year;
{
  register unsigned int N, j;

  N = (unsigned int) (floor(jd) + 1.5 - ISLAMIC_EPOCH);
  
  *year = 1;
  while(N > (10631 * *year + 14)/30 + 1) 
    (*year)++;
  N -= (unsigned int) (islamic2jd(1,1,*year) - ISLAMIC_EPOCH);

  *month = 1;
  while(N > (j = islamic_month_days(*month,*year))){ 
    (*month)++;
    N -= j;
  }

  *day = N;
}


/*-------------------------------------------------------------------------*/
/* Function `islamic_day_of_week'                                           */
/*-------------------------------------------------------------------------*/
int islamic_day_of_week(month,day,year)
     int month, day, year;
{
  unsigned int N;
  
  N = (unsigned int) (islamic2jd(month,day,year) - ISLAMIC_EPOCH) + 4;
  return(N%7);
} 


/*-------------------------------------------------------------------------*/
/* Function `islamic_monthkal'                                             */
/* Formats the calendar for an Islamic month                               */
/*-------------------------------------------------------------------------*/
void islamic_monthkal(month,day,year,color,out)
     int month, day, year, color; 
     FILE *out;
{
  int i, j, N, d, a[6][7];
  char *name;
  
  
  /* Format the head */
  i = 1;
  j = year;
  while(j /= 10) i++;

  name = islamic_monthname[month - 1];
  N = 19 - strlen(name) - i;
  N = N%2 ? (N+1)/2 : N/2;
  fprintf(out,"%s%s %i\n", void_string(N), name, year);
  fprintf(out,"D1 D2 D3 D4 D5 D6 D7\n");

  /* Format the month matrix */
  N = 1;
  d = islamic_day_of_week(month,1,year);
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      a[i][j] = N - d;
      N++;
    }
  }

  N = islamic_month_days(month,year);
  for(i = 0; i < 6; i++){
    for(j = 0; j < 7; j++){
      if(a[i][j] < 1 || a[i][j] > N) fprintf(out,"   ");
      else
	if(a[i][j] != day) 
	  fprintf(out,"%2i ", a[i][j]);
	else 
	  fprintf(out,"%s%2i%s ",MODE_R1(color),a[i][j],MODE_N(color));
    }
    fprintf(out,"\n");
  }
  fprintf(out,"\n");
}


/*-------------------------------------------------------------------------*/
/* Function `islamic_yearkal_head'                                         */
/* Header for a Hebrew year's calendar                                     */
/*-------------------------------------------------------------------------*/
void islamic_yearkal_head(k,out)
     int k; 
     FILE *out;
{
  int l, N;
  char *name[3];

  for(l = 0; l < 3; l++){
    name[l] = islamic_monthname[3*k + l]; 

    N = 20 - strlen(name[l]);
    N = N%2 ? (N+1)/2 : N/2;
    fprintf(out,"%s%s%s   ",void_string(N),name[l],
	    void_string(20 - N - strlen(name[l]))); 
  }
  fprintf(out,"\n");
  for(l = 0; l < 3; l++)  
    fprintf(out,"D1 D2 D3 D4 D5 D6 D7   ");
  fprintf(out,"\n");
}


/*-------------------------------------------------------------------------*/
/* Function `islamic_yearkal'                                              */
/* Formatted output of an Islamic year's calendar                          */
/*-------------------------------------------------------------------------*/
void islamic_yearkal(year,out)
     int year;
     FILE *out;
{
  int i, j, k, l, N, d, a[6][7][3];
  
  /* Year as title */
  i = 1;
  j = year;
  while(j /= 10) i++;
  N = 66 - i;
  N = N%2 ? (N + 1)/2 : N/2;
  fprintf(out,"%s%i\n\n", void_string(N), year);

  /* Format the block of month matrices with headers */
  for(k = 0; k < 4; k++){
    islamic_yearkal_head(k,out);

    for(l = 0; l < 3; l++){
      N = 1;
      d = islamic_day_of_week(3*k + 1 + l,1,year);
      for(i = 0; i < 6; i++){
	for(j = 0; j < 7; j++){
	  a[i][j][l] = N - d;
	  N++;
	}
      }
    }

    for(i = 0; i < 6; i++){
      for(l = 0; l < 3; l++){
	N = islamic_month_days(3*k + 1 + l,year);
	for(j = 0; j < 7; j++){
	  if(a[i][j][l] < 1 || a[i][j][l] > N) fprintf(out,"   ");
	  else fprintf(out,"%2i ", a[i][j][l]);
	}
	fprintf(out,"  ");
      }
      fprintf(out,"\n");
    }
  }
  fprintf(out,"\n");
}


/*-------------------------------------------------------------------------*/
/* Function `print_islamic_day'                                            */
/* Takes a Julian date as input and prints the corresponding Islamic date  */
/*-------------------------------------------------------------------------*/
void print_islamic_day(jd,color,out)
     double jd;
     int color; 
     FILE *out;
{
  int month, day, year;
  
  jd2islamic(jd,&month,&day,&year);
  fprintf(out,"Islamic:\t");
  fprintf(out,"%s%s",MODE_G1(color),islamic_monthname[month - 1]);
  fprintf(out," %i %i%s",day,year,MODE_N(color));
  fprintf(out,"  (%02i/%02i/%i)",month,day,year);
  day = ((unsigned int)(jd - ISLAMIC_EPOCH) + 5)%7;
  switch(day){
  case 0:
    fprintf(out,"  Day 7\n");
    break;
  case 6:
    fprintf(out,"  Jum'a\n");
    break;
  default:
    fprintf(out,"  Day %i\n",day);
  }
}
