/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* kal.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include "kal.h"
#include "latex.h"
#include "moon.h"

/* Some global settings */
int shift = SHIFT, sunday = SUNDAY, extra = EXTRA;
char *prog;

char *monthname[] = {"January","February","March","April","May",
		     "June","July","August","September","October",
		     "November","December"};


char *hebrew_monthname_0[] = {"Tishri","Heshvan","Kislev",
			      "Tevet","Shevat","Adar",
			      "Nisan","Iyyar","Sivan",
			      "Tammuz","Av","Elul"};

char *hebrew_monthname_1[] = {"Tishri","Heshvan","Kislev",
			      "Tevet","Shevat","Adar I","Adar II",
			      "Nisan","Iyyar","Sivan",
			      "Tammuz","Av","Elul"};


char *islamic_monthname[] = {"Muharram","Safar","Rabi'a I",
			     "Rabi'a II","Jumada I","Jumada II",
			     "Rajab","Sha'ban","Ramadan",
			     "Shawwal","Dhu al-Q'adah","Dhu al-Hijjah"};


/*-------------------------------------------------------------------------*/
/* Function `error_test'                                                   */
/* Testing date format                                                     */
/*-------------------------------------------------------------------------*/
void error_test(month,day,year,out)
     int month, day, year; 
     FILE *out;
{
  
  if(year < 1 || year > 9999){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal year value: %i (use 1-9999)\n",prog,year);
    exit(1);
  }
  
  if(month < 1 || month > 12){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal month value: %i (use 1-12)\n",prog,month);
    exit(1);
  }

  if(day < 1 || day > mlength(month,year)){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal day value: %i\n",prog,day);
    exit(1);
  }
  
  if(extra){
    if(year == 1752 && month == 9 && (day >= 3 && day <= 13)){
      if(out != stdout) pclose(out); 
      fprintf(stderr,"%s: non-existing date: %i/%i/%i ",prog,month,day,year); 
      fprintf(stderr,"Gregorian Reformation)\n");
      exit(1);
    }
  }
  else {
    if(year == 1582 && month == 10 && (day >= 5 && day <= 14)){
      if(out != stdout) pclose(out); 
      fprintf(stderr,"%s: non-existing date: %i/%i/%i ",prog,month,day,year);
      fprintf(stderr,"(Gregorian Reformation)\n");
      exit(1);
    }   
  }
}

/*-------------------------------------------------------------------------*/
/* Function `error_test_plus'                                              */
/* Error conditions for `days_plus'                                        */
/*-------------------------------------------------------------------------*/
void error_test_plus(month,day,year,num,out)
     int month, day, year, num; 
     FILE *out;
{
  int a = num + daynum(day,month,year);
  
  if(a < 1 || a > daynum(31,12,9999)){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: result is not a date between January 1 1 ",prog);
    fprintf(stderr,"and December 31 9999\n");
    exit(1);
  }
}

/*-------------------------------------------------------------------------*/
/* Function `numeric_error_test'                                           */
/* Error conditions for `numeric'                                          */
/*-------------------------------------------------------------------------*/
int numeric_error_test(num,year,out)
     int num, year;
     FILE *out;
{
  int maxday;

  maxday = ((extra && year == 1752) || (!extra && year == 1582)) ? 365 :
    daynum(12,31,year) - daynum(1,1,year) + 1;

  if(num > maxday){
    fprintf(out,"%s: Numerical date exceeds %i:%i\n",
            prog,maxday,year);
    return(1);
  }

  if((extra && year == 1752 && num > 246 && num < 258) ||
     (!extra && year == 1582 && num > 277 && num < 288)){
    fprintf(out,"%s: non-existing numerical date: ",prog);
    fprintf(out,"%i:%i (Gregorian Reformation)\n",num,year);
    return(1);
  }
  return(0);
}

/*-------------------------------------------------------------------------*/
/* Function `iso_week_error_test'                                          */
/* Error conditions for `iso_week'                                         */
/*-------------------------------------------------------------------------*/
int iso_week_error_test(week,year,out)
     int week, year; 
     FILE *out;
{
  int maxweek = kal_week(12,31,year,1);

  if(maxweek == 1) maxweek = kal_week(12,24,year,1);

  if(week > maxweek){
    fprintf(out,"%s: The year %i has %i ISO weeks\n",prog,year,maxweek);
    return(1);
  }

  return(0);
}

/*-------------------------------------------------------------------------*/
/* Function `jd_error_test'                                                */
/* Testing Julian date                                                     */
/*-------------------------------------------------------------------------*/
void jd_error_test(jd,out)
     double jd; 
     FILE *out;
{
  
  if(jd < JMIN || jd > JMAX){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal format ",prog);
    fprintf(stderr,"or unsupported Julian date ");
    fprintf(stderr,"(use [%.1f,%.1f])\n",JMIN,JMAX);
    exit(1);
  }
}

/*-------------------------------------------------------------------------*/
/* Function `hebrew_error_test'                                            */
/* Testing Hebrew date                                                     */
/*-------------------------------------------------------------------------*/
void hebrew_error_test(month,day,year,out)
     int month, day, year;
     FILE *out;
{
  
  if(year < 1 || year > 13760){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal Hebrew year value: %i (use 1-13760)\n",
	    prog,year);
    exit(1);
  }

  if(month < 1 || month > hebrew_year_months(year)){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal Hebrew month: %i/%i\n",prog,month,year);
    exit(1);
  }

  if(day < 1 || day > hebrew_month_days(month,year)){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal Hebrew day: %i/%i/%i\n",prog,month,day,year);
    exit(1);
  }
}

/*-------------------------------------------------------------------------*/
/* Function `islamic_error_test'                                           */
/* Testing Islamic date                                                    */
/*-------------------------------------------------------------------------*/
void islamic_error_test(month,day,year,out)
     int month, day, year;
     FILE *out;
{
  
  if(year < 1 || year > 9665){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal Islamic year value: %i (use 1-9665)\n",
	    prog,year);
    exit(1);
  }

  if(month < 1 || month > 12){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal Islamic month: %i\n",prog,month);
    exit(1);
  }

  if(day < 1 || day > islamic_month_days(month,year)){
    if(out != stdout) pclose(out); 
    fprintf(stderr,"%s: illegal Islamic day: %i/%i/%i\n",prog,month,day,year);
    exit(1);
  }
}


/*-------------------------------------------------------------------------*/
/* Function `set_filter'                                                   */
/*-------------------------------------------------------------------------*/
FILE *set_filter(filter,which)
     char *filter; 
     int which;
{
  FILE *pipe;

  if(which){
    switch(which){
    case 2:
      sprintf(filter,"%s ",LESS);
      break;
    case 3:
      sprintf(filter,"%s ",MOST);
      break;
    default:
      sprintf(filter,"%s ",MORE);
    }
    
    if((pipe = popen(filter,"w")) == NULL){
      fprintf(stderr,"%s: Can't open pipe.\n",prog);
      exit(1);
    }
    return(pipe);
  }
  else{
    *filter = 1;
    return(stdout);
  }
}

/*-------------------------------------------------------------------------*/
/* Function `usage'                                                        */
/*-------------------------------------------------------------------------*/
void usage()
{
  printf("Try option `-h' or `-help' for more information.\n");
}

/*-------------------------------------------------------------------------*/
/* Function `help'                                                         */
/*-------------------------------------------------------------------------*/
void help(out)
     FILE *out;
{
  fprintf(out,"Usage: kal [arguments]\n\n" 
	  "The argument types are as follows:\n"
	  "\nStandard functions\n"
	  "  [month/[day/]]year\n"
	  "  [[num]:][year]\tNumerical date format to standard format\n"
	  "  [[week].][year]\tDisplays ISO week\n"
	  "  -y[year]\t\tCalendar of the current year or specified year\n"
	  "\nStandard functions for Hebrew calendar\n"
	  "  -j[[month/[day/]]year]\t(Hebrew calendar dates expected)\n"
	  "\nStandard functions for Islamic calendar\n"
	  "  -i[[month/[day/]]year]\t(Islamic calendar dates expected)\n"
	  "\nCalendar Arithmetics\n"
	  "  Use option `-j\' for Hebrew calendar dates:\n"
	  "  [-j]month/day/year+|-num\t"
	  "Date month/day/year plus|minus num days\n"
	  "  [-j][%%]m1/d1/y1,[m2/d2/y2]\tDifference of dates\n"
	  "\nMoon\n"
	  "  -m[month/day/year]\t"
	  "Moon phase for the date month/day/year\n"
	  "  -m[juliandate]\tMoon phase for the specified Julian date\n"
	  "\nHolidays\n"
	  "  -E year\t\tEaster Sunday of year\n"
	  "  -G year\t\tGreek Orthodox Easter Sunday of year\n"
	  "  -J year\t\tJewish New Year of (Gregorian) year\n" 
	  "\nExport\n"
	  "  -o [month/]year\tPrint HTML calendar to stdout\n"
	  "  -p [month/]year\tPrint HTML calendar to file\n"
	  "  -t [month/]year\tPrint TeX calendar\n"
	  "\nOther\n"
	  "  -s=sitename\tSelect geographic location from sites file\n"
	  "  -l=style\tSelect calendar layout style \"us\" or \"eu\"\n"
	  "  -g=year\tSelect year \"1582\" or \"1752\" of "
	  "Gregorian Reformation\n" 
	  "  -f=value\tSelect output filter\n"
	  "\t\t(use value 0 for none, 1 for more, 2 for less, 3 for most)\n"
	  "  -c\t\tColor toggle\n"
	  "  -v\t\tVersion information\n"
	  "  -h\t\tThis help message\n"
	  "\nList of options and corresponding long options:\n"
	  "  -y, --year\n  -m, --moon\n  -s, --site\n  -j, --jewish\n"
	  "  -i, --islamic\n  -E, --EasterSunday\n"
	  "  -G, --GreekOrthodoxEasterSunday\n"
	  "  -J, --JewishNewYear\n  -o, --output\n  -p, --print\n"
	  "  -t, --tex\n  -l, --layout\n  -g, --gregorian\n  -f, --filter\n"
	  "  -c, --color-toggle\n  -v, --version\n  -h, --help\n"
	  "\n");
}


/*-------------------------------------------------------------------------*/
/* Function `license'                                                      */
/*-------------------------------------------------------------------------*/
void license(out)
     FILE *out;
{
  fprintf(out,"\n"
	  "kal is free software and you are welcome to distribute copies"
	  " of it\nunder certain conditions. There is ABSOLUTELY NO "
	  "WARRANTY for kal.\nFor full details, see the file COPYING "
	  "(shipped along with this program),\nor write to:\n"
	  "\tFree Software Foundation, Inc.\n"
	  "\t675 Mass Ave\n\tCambridge, MA 02139\n\tUSA\n\n");
}


/*-------------------------------------------------------------------------*/
/* Main program                                                            */
/*-------------------------------------------------------------------------*/
int main(argc,argv)
     int argc; 
     char **argv;
{
  int c, d, ind, opt, arg1, arg2, arg3, arg4, arg5, arg6, color = COLOR,
    longind, flag = 0, flag2 = 1;
  double jd;
  char *options = "cE:G:o:p:t:i::j::J:m::s:y::f:l:g:hv", *ptr, *ptr1, *ptr2, 
    *filter = NULL, *duparg, *sitename;
  FILE *out = stdout;
  time_t now;
  struct tm *t;
  struct option longoptions[] = {{"year",2,NULL,'y'},
				 {"moon",2,NULL,'m'},
				 {"site",1,NULL,'s'},
				 {"jewish",2,NULL,'j'},
				 {"islamic",2,NULL,'i'},
				 {"EasterSunday",1,NULL,'E'},
				 {"GreekOrthodoxEasterSunday",1,NULL,'G'},
				 {"JewishNewYear",1,NULL,'J'},
				 {"output",1,NULL,'o'},
				 {"print",1,NULL,'p'},
				 {"tex",1,NULL,'t'},
				 {"color-toggle",0,NULL,'c'},
				 {"filter",1,NULL,'f'},
				 {"layout",1,NULL,'l'},
				 {"gregorian",1,NULL,'g'},
				 {"version",0,NULL,'v'},
				 {"help",0,NULL,'h'},
				 {NULL,0,NULL,0}};
  

  prog = argv[0];
  filter = (char *) calloc(9,sizeof(char));

  while((opt = getopt_long_only(argc,argv,options,longoptions,&longind)) 
	!= EOF){
    switch(opt){
    case 'c':
      color = (color) ? 0 : 1;
      break;
    case 's':
      sitename = (char *) calloc(strlen(optarg) + 10, sizeof(char));
      sprintf(sitename,"KAL_SITE=%s",optarg);
      putenv(sitename);
      break;
    case 'f':
      if(!strcmp(optarg,"0") || !strcmp(optarg,"1") 
	 || !strcmp(optarg,"2") || !strcmp(optarg,"3"))
	flag = atoi(optarg) - 10;
      else {
	fprintf(stderr,"%s: use `-filter=value' with value 0, 1, 2, or 3\n",
		prog);
	exit(1);
      }
      break;
    case 'l':
      if(!strcmp(optarg,"eu")){
	shift  = 5;
	sunday = 6;
      }
      else 
	if(!strcmp(optarg,"us")){
	  shift  = 6;
	  sunday = 0;
	} 
	else {
	  fprintf(stderr,"%s: use `-layout=style' with style "
		  "\"us\" or \"eu\"\n", prog);
	  exit(1);
	}
      break;
    case 'g':
      if(!strcmp(optarg,"1582"))
	extra = 0;
      else 
	if(!strcmp(optarg,"1752"))
	  extra = 1;
	else {
	  fprintf(stderr,"%s: use `-gregorian=year' with year "
		  "\"1582\" or \"1752\"\n", prog);
	  exit(1);
	}
      break;
    case 'i':
    case 'j':
      flag2 = 0;
      if(optarg){
	c = (strstr(optarg,",")) ? 2 :
	  ((strstr(optarg,"+") || strstr(optarg,"-")) ? 1 : 0);
	if(c == 0){
	  duparg = (char *) calloc(strlen(optarg) + 1, sizeof(char));
	  strcpy(duparg,optarg);
	  arg1 = ((ptr = strtok(duparg,"/"))) ? atoi(ptr) : 0;
	  arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  arg3 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  if(!arg3)
	    if(!arg2){
	      /* arg1 is year */
	      if(flag == 0) flag = FILTER;
	    }
	  free(duparg);
	}
      }
      break;
    default:
      flag2 = 0;
      if(flag == 0 && (opt == 'y' || opt == 'h')) flag = FILTER;
    }
  }
  
  if(argv[optind]){
    flag2 = 0;
    ind = optind;
    while(argv[ind]){
      c = strstr(argv[ind],",") ? 4 :
	(strstr(argv[ind],"+") || strstr(argv[ind],"-")) ? 3 : 
	strstr(argv[ind],":") ? 2 : 
	strstr(argv[ind],".") ? 1 : 0;
      
      if(c == 0){
	duparg = (char *) calloc(strlen(argv[ind]) + 1, sizeof(char));
	strcpy(duparg,argv[ind]);
	arg1 = ((ptr = strtok(duparg,"/"))) ? atoi(ptr) : 0;
	arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	arg3 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	if(!arg3)
	  if(!arg2){
	    /* arg1 is year */
	    if(flag == 0) flag = FILTER;
	  }
	free(duparg);
      }
      ind++; 
    }
  }
  if(flag < 0) flag += 10; 
  out = set_filter(filter,flag);
  /* Disable colors for `most' */
  if(flag == 3) color = 0;

  time(&now);
  t = localtime(&now);
  optind = 0;
  opterr = 0;
  while((opt = getopt_long_only(argc,argv,options,longoptions,&longind)) 
	!= EOF){
    switch(opt){
    case 'c':
    case 'l':
      if(flag2)
	monthkal(t->tm_mon + 1,t->tm_mday,t->tm_year + 1900,0,0,color,out);
      flag2 = 0;
      break;
    case 's':
    case 'f':
    case 'g':
      break;
    case 'm':
      if(!optarg){
	moon(jtime(gmtime(&now)),color,out);
      }
      else {
	c = (strstr(optarg,"/")) ? 1 : 0;
	switch(c){
	case 1:
	  arg1 = ((ptr = strtok(optarg,"/"))) ? atoi(ptr) : 0;
	  arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  arg3 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  if(ptr){
	    error_test(arg1,arg2,arg3,out); 
	    moon(JMIN - 1.0 + daynum(arg1,arg2,arg3),color,out);
	  }
	  else {
	    fprintf(stderr,"%s: illegal input format\n",argv[0]);
	    usage();
	  }
	  break;
	default:
	  jd = atof(optarg);
	  jd_error_test(jd,out);
	  moon(jd,color,out);
	}
      }
      break;
    case 'i':
      if(optarg){
	c = (strstr(optarg,",")) ? 2 :
	  ((strstr(optarg,"+") || strstr(optarg,"-")) ? 1 : 0);
	switch(c){
	case 0:
	  arg1 = ((ptr = strtok(optarg,"/"))) ? atoi(ptr) : 0;
	  arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  arg3 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  if(!arg3)
	    if(!arg2){
	      /* arg1 is year */
	      islamic_error_test(1,1,arg1,out);
	      islamic_yearkal(arg1,out);
	    }
	    else {
	      /* arg1 is month, arg2 is year */
	      islamic_error_test(arg1,1,arg2,out);
	      islamic_monthkal(arg1,0,arg2,color,out);
	    }
	  else {
	    /* arg1 is month, arg2 is day, arg3 is year */
	    islamic_error_test(arg1,arg2,arg3,out);
	    jd = islamic2jd(arg1,arg2,arg3);
	    if(jd >= JMIN && jd < JMAX){
	      inv_julian(jd,&arg4,&arg5,&arg6);
	      print_day(arg4,arg5,arg6,color,out);
	    }
	    print_hebrew_day(jd + 1.0,color,out);
	    print_islamic_day(jd,color,out);
	    fprintf(out,"Julian date:\t");
	    fprintf(out,"%s%.1f%s",MODE_G1(color),jd,MODE_N(color));
	    fprintf(out,"\n");
	    updown(jd, color, out);
	  }
	  break;
	default:
	  if(out != stdout) pclose(out);
	  fprintf(stderr,"%s: Illegal input\n",argv[0]);
	  usage();
	  exit(1);
	}
      }
      else {
	jd2islamic(jtime(t) - 0.25,&arg1,&arg2,&arg3);
	islamic_monthkal(arg1,arg2,arg3,color,out);
      }
      break;
    case 'j':
      if(optarg){
	c = (strstr(optarg,",")) ? 2 :
	  ((strstr(optarg,"+") || strstr(optarg,"-")) ? 1 : 0);
	switch(c){
	case 0:
	  arg1 = ((ptr = strtok(optarg,"/"))) ? atoi(ptr) : 0;
	  arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  arg3 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	  if(!arg3)
	    if(!arg2){
	      /* arg1 is hyear */
	      hebrew_error_test(1,1,arg1,out);
	      hebrew_yearkal(arg1,out);
	    }
	    else {
	      /* arg1 is month, arg2 is year */
	      hebrew_error_test(arg1,1,arg2,out);
	      hebrew_monthkal(arg1,0,arg2,color,out);
	    }
	  else {
	    /* arg1 is month, arg2 is day, arg3 is year */
	    hebrew_error_test(arg1,arg2,arg3,out);
	    jd = hebrew2jd(arg1,arg2,arg3);
	    if(jd >= JMIN && jd < JMAX){
	      inv_julian(jd,&arg4,&arg5,&arg6);
	      print_day(arg4,arg5,arg6,color,out);
	    }
	    print_hebrew_day(jd + 1.0,color,out);
	    if(jd > ISLAMIC_EPOCH && jd < ISLAMIC_MAX)
	      print_islamic_day(jd,color,out);
	    fprintf(out,"Julian date:\t");
	    fprintf(out,"%s%.1f%s",MODE_G1(color),jd,MODE_N(color));
	    fprintf(out,"\n");
	    if(jd >= JMIN && jd < JMAX)
	      updown(jd, color, out);
	  }
	  break;
	case 2:
	  ptr = strstr(optarg,"%");
	  ptr1 = (*optarg == '%') ? optarg + 1 : optarg;
	  ptr2 = strstr(ptr1,",");
	  *ptr2++ = 0;
	  arg1 = ((ptr1 = strtok(ptr1,"/"))) ? atoi(ptr1) : 0;
	  arg2 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;
	  arg3 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;
	
	  arg4 = ((ptr2 = strtok(ptr2,"/"))) ? atoi(ptr2) : 0;
	  arg5 = ((ptr2 = strtok(NULL,"/"))) ? atoi(ptr2) : 0;
	  arg6 = ((ptr2 = strtok(NULL,"/"))) ? atoi(ptr2) : 0;
	  
	  if(!arg1 && !arg2 && !arg3 && arg4 && arg5 && arg6){ 
	    arg1 = t->tm_mon + 1;
	    arg2 = t->tm_mday; 
	    arg3 = t->tm_year + 1900;
	    jd = JMIN - 1.0 + daynum(arg1,arg2,arg3);
	    jd2hebrew(jd,&arg1,&arg2,&arg3);
	  }

	  if(arg1 && arg2 && arg3 && !arg4 && !arg5 && !arg6){ 
	    arg4 = t->tm_mon + 1;
	    arg5 = t->tm_mday; 
	    arg6 = t->tm_year + 1900;
	    jd = JMIN - 1.0 + daynum(arg4,arg5,arg6);
	    jd2hebrew(jd,&arg4,&arg5,&arg6);
	  }

	  if(!arg1 || !arg2 || !arg3 || !arg4 || !arg5 || !arg6){
	    if(out != stdout) pclose(out);
	    fprintf(stderr,"%s: Illegal input format\n",argv[0]);
	    usage();
	    exit(1);
	  }
	  hebrew_error_test(arg1,arg2,arg3,out);
	  hebrew_error_test(arg4,arg5,arg6,out);
	  jd = hebrew2jd(arg1,arg2,arg3) - hebrew2jd(arg4,arg5,arg6);

	  ptr1 = (hebrew_year_months(arg3) == 12) ?
	    hebrew_monthname_0[arg1 - 1] : hebrew_monthname_1[arg1 - 1];
	  ptr2 = (hebrew_year_months(arg6) == 12) ?
	    hebrew_monthname_0[arg4 - 1] : hebrew_monthname_1[arg4 - 1];
	  if(jd <= 0.0){
	    fprintf(out,"%s %i %i - ", ptr1, arg2, arg3);
	    fprintf(out,"%s %i %i: ", ptr2, arg5, arg6); 
	  }
	  else{
	    fprintf(out,"%s %i %i - ", ptr2, arg5, arg6);
	    fprintf(out,"%s %i %i: ", ptr1, arg2, arg3);
	  }
	  if(!ptr)
	    fprintf(out,"%s%i%s day%s\n", MODE_G1(color),
		    EPL2((unsigned int) fabs(jd),MODE_N(color)));
	  else
	    if(jd <= 0.0)
	      hebrew_fmt_days_between(arg1,arg2,arg3,arg4,arg5,arg6,color,out);
	    else
	      hebrew_fmt_days_between(arg4,arg5,arg6,arg1,arg2,arg3,color,out);
	  break;
	case 1:
	  if((ptr2 = strstr(optarg,"+"))){
	    *ptr2 = 0;
	    ptr1 = strstr(optarg,"-");
	    *ptr2 = '+';
	    if(ptr1) ptr2 = ptr1;
	  }
	  else
	    ptr2 = strstr(optarg,"-");
	
	  arg1 = ((ptr1 = strtok(optarg,"/"))) ? atoi(ptr1) : 0;
	  arg2 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;
	  arg3 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;
	  ptr1 = ptr2 + 1;
	  d    = atoi(ptr1);
	
	  if(!arg1 || !arg2 || !arg3){
	    if(out != stdout) pclose(out);
	    fprintf(stderr,"%s: Illegal input format\n",argv[0]);
	    usage();
	    exit(1);
	  }
	  hebrew_error_test(arg1,arg2,arg3,out);
	  
	  jd = hebrew2jd(arg1,arg2,arg3) + 0.5;
	  jd += (*ptr2 == '+') ? d : -d;
	  if(jd < HEBREW_EPOCH + 2 || jd > HEBREW_MAX){
	    if(out != stdout) pclose(out);
	    fprintf(stderr,"%s: Result not in supported range\n",argv[0]);
	    exit(1);
	  }
	  jd2hebrew(jd,&arg4,&arg5,&arg6);
	  ptr1 = (hebrew_year_months(arg3) == 12) ?
	    hebrew_monthname_0[arg1 - 1] : hebrew_monthname_1[arg1 - 1];
	  
	  if(*ptr2 == '+')
	    fprintf(out,"%s %i %i plus %i day%s: ",ptr1, arg2, arg3, EPL(d));
	  else
	    fprintf(out,"%s %i %i minus %i day%s: ",ptr1, arg2, arg3, EPL(d));
	  ptr1 = (hebrew_year_months(arg4) == 12) ?
	      hebrew_monthname_0[arg4 - 1] : hebrew_monthname_1[arg4 - 1];
	  fprintf(out,"%s%s %i %i%s",
		  MODE_G1(color),ptr1,arg5,arg6,MODE_N(color));
	  if((d = ((unsigned int)(jd - HEBREW_EPOCH))%7))
	    fprintf(out,"  (%02i/%02i/%i)  Day %i\n",arg4,arg5,arg6,d);
	  else 
	    fprintf(out,"  (%02i/%02i/%i)  Sabbath\n",arg4,arg5,arg6);
	  break;
	default:
	  if(out != stdout) pclose(out);
	  fprintf(stderr,"%s: Illegal input\n",argv[0]);
	  usage();
	  exit(1);
	}
      }
      else {
	jd2hebrew(jtime(t) + 0.75,&arg1,&arg2,&arg3);
	hebrew_monthkal(arg1,arg2,arg3,color,out);
      }
      break;
    case 'J':
      arg1 = atoi(optarg);
      if(arg1 < 1 || arg1 > 9999){
	if(out != stdout) pclose(out);
	fprintf(stderr,"%s: use `-J' with year values 1-9999\n\n",argv[0]);
	exit(1);
      }
      RoshHaShanah(arg1,color,out);
      break;
    case 'E':
      arg1 = atoi(optarg);
      error_test(1,1,arg1,out);
      easter_sunday(arg1,color,out);
      break;
    case 'G':
      arg1 = atoi(optarg);
      error_test(1,1,arg1,out);
      easter_sunday_greek(arg1,color,out);
      break;
    case 'y':
      if(!optarg)
	yearkal(t->tm_year + 1900,out);
      else {
	arg1 = atoi(optarg);
	error_test(1,1,arg1,out);
	yearkal(arg1,out);
      }
      break;
    case 'o':
    case 'p':
      arg1 = ((ptr = strtok(optarg,"/"))) ? atoi(ptr) : 0;
      arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
      if(!arg2){
	/* arg1 is year */
	error_test(1,1,arg1,out);
	html_days_of_week();
	kal2html(arg1,opt,out);
      }
      else {
	/* arg1 is month, arg2 is year */
	error_test(arg1,1,arg2,out);
	html_days_of_week();
	monthkal2html(arg1,arg2,opt,out);
      }
      break;
    case 't':
      arg1 = ((ptr = strtok(optarg,"/"))) ? atoi(ptr) : 0;
      arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
      if(!arg2){
	/* arg1 is year */
	error_test(1,1,arg1,out);
	kal2tex(arg1,out);
      }
      else {
	/* arg1 is month, arg2 is year */
	error_test(arg1,1,arg2,out);
	monthkal2tex(arg1,arg2,out);
      }
      break;
    case 'h':
      help(out);
      break;
    case 'v':
      fprintf(out,"kal %s\n", VERSION);
      fprintf(out,"Active flags:\n");
      fprintf(out,"\tCalendar style is ");
      if(SUNDAY) 
	fprintf(out,"European\n");
      else 
	fprintf(out,"American\n");
      fprintf(out,"\tGregorian Reformation is assumed in ");
      if(EXTRA)
	fprintf(out,"September 1752\n");
      else
	fprintf(out,"October 1582\n");
      fprintf(out,"\tLaTeX output uses \"%s\"\n",PAPERSIZE);
      fprintf(out,"\tUse colors by default: ");
      if(COLOR)
	fprintf(out,"Yes\n");
      else
	fprintf(out,"No\n");
      fprintf(out,"\tUse default filter: ");
      switch(FILTER){
      case 1:
	fprintf(out,"%s\n",MORE);
	break;
      case 2:
	fprintf(out,"%s\n",LESS);
	break;
      case 3:
	fprintf(out,"%s\n",MOST);
	break;
      default:
	fprintf(out,"none\n");
      }
      fprintf(out,"\nSystem sites file:\n\t%s\n",SITES_SYS);
      license(out); 
      break;
    default:
      if(out != stdout) pclose(out);
      usage();
      exit(1);
    }
  }
  
  if(argv[optind]){
    ind = optind;
    while(argv[ind]){
      c = strstr(argv[ind],",") ? 4 :
	(strstr(argv[ind],"+") || strstr(argv[ind],"-")) ? 3 : 
	strstr(argv[ind],":") ? 2 : 
	strstr(argv[ind],".") ? 1 : 0;

      switch(c){
      case 0:
	arg1 = ((ptr = strtok(argv[ind],"/"))) ? atoi(ptr) : 0;
	arg2 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	arg3 = ((ptr = strtok(NULL,"/"))) ? atoi(ptr) : 0;
	if(!arg3)
	  if(!arg2){
	    /* arg1 is year */
	    error_test(1,1,arg1,out);
	    yearkal(arg1,out);
	  }
	  else {
	    /* arg1 is month, arg2 is year */
	    error_test(arg1,1,arg2,out);
	    monthkal(arg1,0,arg2,0,0,color,out);
	  }
	else {
	  /* arg1 is month, arg2 is day, arg3 is year */
	  error_test(arg1,arg2,arg3,out);
	  print_day(arg1,arg2,arg3,color,out);
	  jd = JMIN + daynum(arg1,arg2,arg3);
	  print_hebrew_day(jd,color,out);
	  if(jd > ISLAMIC_EPOCH + 1.0 && jd < ISLAMIC_MAX + 1.0)
	    print_islamic_day(jd - 1.0,color,out);
	  fprintf(out,"Julian date:\t");
	  fprintf(out,"%s%.1f%s",
		  MODE_G1(color),JMIN - 1.0 + daynum(arg1,arg2,arg3),
		  MODE_N(color));
	  fprintf(out,"\n");
	  updown(JMIN - 0.5 + daynum(arg1,arg2,arg3),color,out);
	}
	break;
      case 4: 
	ptr = strstr(argv[ind],"%");
	ptr1 = (*argv[ind] == '%') ? argv[ind] + 1 : argv[ind];
	ptr2 = strstr(ptr1,",");
	*ptr2++ = 0;
	arg1 = ((ptr1 = strtok(ptr1,"/"))) ? atoi(ptr1) : 0;
	arg2 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;
	arg3 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;

	arg4 = ((ptr2 = strtok(ptr2,"/"))) ? atoi(ptr2) : 0;
	arg5 = ((ptr2 = strtok(NULL,"/"))) ? atoi(ptr2) : 0;
	arg6 = ((ptr2 = strtok(NULL,"/"))) ? atoi(ptr2) : 0;

	if(!arg1 && !arg2 && !arg3 && arg4 && arg5 && arg6){ 
	  arg1 = t->tm_mon + 1;
	  arg2 = t->tm_mday; 
	  arg3 = t->tm_year + 1900;
	}

	if(arg1 && arg2 && arg3 && !arg4 && !arg5 && !arg6){ 
	  arg4 = t->tm_mon + 1;
	  arg5 = t->tm_mday; 
	  arg6 = t->tm_year + 1900;
	}
	
	if(!arg1 || !arg2 || !arg3 || !arg4 || !arg5 || !arg6){
	  if(out != stdout) pclose(out);
	  fprintf(stderr,"%s: Illegal input format\n",argv[0]);
	  usage();
	  exit(1);
	}
	error_test(arg1,arg2,arg3,out);
	error_test(arg4,arg5,arg6,out);
	d = daynum(arg1,arg2,arg3) - daynum(arg4,arg5,arg6);

	if(d <= 0){
	  fprintf(out,"%s %i %i - ", monthname[arg1 - 1], arg2, arg3);
	  fprintf(out,"%s %i %i: ", monthname[arg4 - 1], arg5, arg6); 
	}
	else{
	  fprintf(out,"%s %i %i - ", monthname[arg4 - 1], arg5, arg6);
	  fprintf(out,"%s %i %i: ", monthname[arg1 - 1], arg2, arg3);
	}
	if(!ptr)
	  fprintf(out,"%s%i%s day%s\n", MODE_G1(color),
		  EPL2(abs(d),MODE_N(color)));
	else 
	  if(d <= 0)
	    fmt_days_between(arg1,arg2,arg3,arg4,arg5,arg6,color,out);
	  else
	    fmt_days_between(arg4,arg5,arg6,arg1,arg2,arg3,color,out);
	break;
      case 3:
	if((ptr2 = strstr(argv[ind],"+"))){
	  *ptr2 = 0;
	  ptr1 = strstr(argv[ind],"-");
	  *ptr2 = '+';
	  if(ptr1) ptr2 = ptr1;
	}
	else
	  ptr2 = strstr(argv[ind],"-");
	
	arg1 = ((ptr1 = strtok(argv[ind],"/"))) ? atoi(ptr1) : 0;
	arg2 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;
	arg3 = ((ptr1 = strtok(NULL,"/"))) ? atoi(ptr1) : 0;
	ptr1 = ptr2 + 1;
	d    = atoi(ptr1);
	
	if(!arg1 || !arg2 || !arg3){
	  if(out != stdout) pclose(out);
	  fprintf(stderr,"%s: Illegal input format\n",argv[0]);
	  usage();
	  exit(1);
	}
	error_test(arg1,arg2,arg3,out);
	
	if(*ptr2 == '+'){
	  error_test_plus(arg1,arg2,arg3,d,out);
	  fprintf(out,"%s %i %i plus %i day%s: ", 
		 monthname[arg1 - 1], arg2, arg3, EPL(d));
	  days_plus(arg1,arg2,arg3,d,color,out);
	}
	if(*ptr2 == '-'){
	  error_test_plus(arg1,arg2,arg3,-d,out);
	  fprintf(out,"%s %i %i minus %i day%s: ", 
		 monthname[arg1 - 1], arg2, arg3, EPL(d));
	  days_plus(arg1,arg2,arg3,-d,color,out);
	}
	break;
      case 2:
	arg1 = ((ptr1 = strtok(argv[ind],":"))) ? atoi(ptr1) : 0;
	arg2 = ((ptr1 = strtok(NULL,":"))) ? atoi(ptr1) : 0;
	if(!arg1){
	  arg1 = t->tm_yday + 1;
	  arg2 = t->tm_year + 1900;
	}
	else 
	  if(!arg2) 
	    arg2 = t->tm_year + 1900;

	error_test(1,1,arg2,out);
	if(arg1 < 1){
	  if(out != stdout) pclose(out); 
	  fprintf(out,"%s: Illegal day number: %i\n",argv[0],arg1);
	  exit(1);
	}
	if(!numeric_error_test(arg1,arg2,out)) numeric(arg1,arg2,color,out);
	break;
      case 1:
	arg1 = ((ptr1 = strtok(argv[ind],"."))) ? atoi(ptr1) : 0;
	arg2 = ((ptr1 = strtok(NULL,"."))) ? atoi(ptr1) : 0;
	if(!arg1){
	  arg2 = t->tm_year + 1900;
	  arg1 = kal_week(t->tm_mon + 1,t->tm_mday,arg2,1);
	}
	else 
	  if(!arg2) 
	    arg2 = t->tm_year + 1900;

	error_test(1,1,arg2,out);
	if(arg1 < 1){
	  if(out != stdout) pclose(out); 
	  fprintf(out,"%s: Illegal week number: %i\n",argv[0],arg1);
	  exit(1);
	}
	if(!iso_week_error_test(arg1,arg2,out)) iso_week(arg1,arg2,color,out);
	break;
      default:
	if(out != stdout) pclose(out);
	fprintf(stderr,"%s: Illegal input\n",argv[0]);
	usage();
	exit(1);
      }
     ind++; 
    }
  }
  else
    if(optind == 1){
      monthkal(t->tm_mon + 1, t->tm_mday, t->tm_year + 1900,0,0,color,out);
    }

  if(out != stdout)
    pclose(out);

  return(0);
}
