/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
   Adopted from xmoontool by John Walker. 
   Most of the code in moon.c as well as the content of the header
   file moon.h has been extracted from xmoontool.c.  

   Xmoontool is in the public domain so that the conditions of the 
   GNU General Public License do not apply to the original code.
   Xmoontool can be downloaded from http://www.fourmilab.ch/

   For the numerical methods and additional information see the 
   following note from xmoontool.c:
*/

/*************************************************************************

    The algorithms used in this program to calculate the positions Sun and
    Moon as seen from the Earth are given in the book "Practical Astronomy
    With  Your  Calculator"  by  Peter  Duffett-Smith,   Second   Edition,
    Cambridge University Press, 1981.  Ignore the word "Calculator" in the
    title;  this  is  an  essential  reference  if  you're  interested  in
    developing	software  which  calculates  planetary	positions, orbits,
    eclipses, and  the  like.   If  you're  interested  in  pursuing  such
    programming, you should also obtain:

    "Astronomical  Formulae for Calculators" by Jean Meeus, Third Edition,
    Willmann-Bell, 1985.  A must-have.

    "Planetary  Programs  and  Tables  from  -4000  to  +2800"  by  Pierre
    Bretagnon  and Jean-Louis Simon, Willmann-Bell, 1986.  If you want the
    utmost  (outside  of  JPL)  accuracy  for  the  planets,  it's   here.

    "Celestial BASIC" by Eric Burgess, Revised Edition, Sybex, 1985.  Very
    cookbook oriented, and many of the algorithms are hard to dig  out	of
    the turgid BASIC code, but you'll probably want it anyway.

    Many of these references can be obtained from Willmann-Bell, P.O.  Box
    35025,  Richmond,  VA 23235, USA.  Phone: (804) 320-7016.  In addition
    to their own publications, they stock most of the standard	references
    for mathematical and positional astronomy.

    This program was written by:

	John Walker
	kelvin@fourmilab.ch
	http://www.fourmilab.ch/

    This  program is in the public domain: "Do what thou wilt shall be the
    whole of the law".  I'd appreciate  receiving  any  bug  fixes  and/or
    enhancements,  which  I'll  incorporate  in  future  versions  of  the
    program.  Please leave the original attribution information intact	so
    that credit and blame may be properly apportioned.

**************************************************************************/

/* moon.c */

#include <stdio.h>
#include <stdlib.h>
#include "kal.h"
#include "moon.h"


extern int extra;


static double nptime = 0.0;	      /* Next new moon time */

static char *labels[] = {"Julian date:","Universal time:","Local time:",
			 "","Age of moon:","Moon phase:",
			 "Moon's distance:","Moon subtends:","",
			 "Sun's distance:","Sun subtends:","",
			 "Last new moon:","First quarter:","Full moon:",
			 "Last quarter:","Next new moon:"};

static char *phasname[] = {"new moon","wax moon","full moon","wane moon"};

static char *moname[] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug",
			 "Sep","Oct","Nov","Dec"};



/*-------------------------------------------------------------------------*/
/* Function `inv_julian':                                                  */
/* Converts date format julian -> standard (suitable for kal)              */
/*-------------------------------------------------------------------------*/
void inv_julian(td,mm,dd,yy)
     double td; 
     int *mm, *dd, *yy;
{
  register int j;

  td += 0.5;			      /* Astronomical to civil */
  j = td; 
    
  if((extra && j > 2361221.5) || (!extra && j > 2299160.5)) 
    jyear(td - 0.5, yy, mm,dd);
  else{
    *yy = 1;
    *mm = 1;
    j -= (JMIN - 0.5);  
    if(j > 0){
      while(j > daynum(12,31,*yy)) (*yy)++;
      j -= daynum(1,1,*yy) - 1;
      while(j > mlength(*mm,*yy))
	j -= mlength((*mm)++,*yy);

      *dd = j;
    }
    else{
      *yy = 0;
      *mm = 12;
      *dd = 31 + j;
    }
  }
}


/*  FMT_PHASE_TIME  --	Format	the  provided  julian  date  into  the
			provided  buffer  in  UTC  format  for  screen
			display  */

void fmt_phase_time(utime,i,lun,loc,zname,offset,out)
     double utime;
     int i, lun; 
     char *loc, *zname, *offset; 
     FILE *out;
{
  int yy, mm, dd, hh, mmm, ss;
  double ltime;

  if((ltime = localmoon(utime,loc,zname,offset)) != -1.0 && ltime != -2.0){
    inv_julian(ltime, &mm, &dd, &yy);
    jhms(ltime, &hh, &mmm, &ss);
  }
  else {
    inv_julian(utime, &mm, &dd, &yy);
    jhms(utime, &hh, &mmm, &ss);
    sprintf(zname,"GMT");
  }

  fprintf(out,"%s\t%2i:%02i  %s %2i %4i  %s",
	  labels[12+i], hh, mmm, moname[mm - 1], dd, yy, zname);
  switch (i){
  case 0:
  case 4:
    fprintf(out,"  \t Lunation: %i\n", lun + (i % 3));
    break;
  default:
    fprintf(out,"\n");
  }
}


/*  JDATE  --  Convert internal GMT date and time to  Julian  day  and
	       fraction.  */

int jdate(t)
     struct tm *t;
{
  int c, m, y;
  
  y = t->tm_year + 1900;
  m = t->tm_mon + 1;
  
  if(m > 2){
    m = m - 3;
  } 
  else{
    m = m + 9;
    y--;
  }
    
  c = y / 100L;		      /* Compute century */
  y -= 100L * c;
  return (t->tm_mday + (c * 146097L) / 4 + (y * 1461L) / 4 +
	  (m * 153L + 2) / 5 + 1721119L);
}


/*  JTIME  --  Convert internal GMT  date  and	time  to  astronomical
	       Julian	time  (i.e. Julian  date  plus	day  fraction,
	       expressed as a double).	*/

double jtime(t)
     struct tm *t;
{
  return (jdate(t) - 0.5) + 
    (t->tm_sec + 60 * (t->tm_min + 60 * t->tm_hour)) / 86400.0;
}


/*  JYEAR  --  Convert	Julian	date  to  year,  month, day, which are
	       returned via integer pointers to integers.  */

void jyear(td,yy,mm,dd)
     double td; 
     int *yy, *mm, *dd;
{
  double j, d, y, m;

  td += 0.5;			      /* Astronomical to civil */
  j = floor(td);
  j = j - 1721119.0;
  y = floor(((4 * j) - 1) / 146097.0);
  j = (j * 4.0) - (1.0 + (146097.0 * y));
  d = floor(j / 4.0);
  j = floor(((4.0 * d) + 3.0) / 1461.0);
  d = ((4.0 * d) + 3.0) - (1461.0 * j);
  d = floor((d + 4.0) / 4.0);
  m = floor(((5.0 * d) - 3) / 153.0);
  d = (5.0 * d) - (3.0 + (153.0 * m));
  d = floor((d + 5.0) / 5.0);
  y = (100.0 * y) + j;
  if(m < 10.0){
    m = m + 3;
  }
  else{
    m = m - 9;
    y = y + 1;
  }
  *yy = y;
  *mm = m;
  *dd = d;
}


/*  JHMS  --  Convert Julian time to hour, minutes, and seconds.  */

void jhms(j,h,m,s)
     double j; 
     int *h, *m, *s;
{
  long ij;

  j += 0.5;			      /* Astronomical to civil */
  ij = rint((j - floor(j)) * 86400.0);
  *h = ij / 3600L;
  *m = (ij / 60L) % 60L;
  *s = ij % 60L;
}


/*  MEANPHASE  --  Calculates  time  of  the mean new Moon for a given
		   base date.  This argument K to this function is the
		   precomputed synodic month index, given by:
  
			  K = (year - 1900) * 12.3685
  
		   where year is expressed as a year and fractional year.  */

double meanphase(sdate,k)
     double sdate, k;
{
  double t, t2, t3, nt1;
  
  /* Time in Julian centuries from 1900 January 0.5 */
  t = (sdate - 2415020.0) / 36525;
  t2 = t * t; 		      /* Square for frequent use */
  t3 = t2 * t;		      /* Cube for frequent use */
  
  nt1 = 2415020.75933 + synmonth * k
        + 0.0001178 * t2
        - 0.000000155 * t3
        + 0.00033 * dsin(166.56 + 132.87 * t - 0.009173 * t2);

  return nt1;
}


/*  TRUEPHASE  --  Given a K value used to determine the mean phase of
		   the new moon, and a phase selector (0.0, 0.25, 0.5,
		   0.75), obtain the true, corrected phase time.  */

double truephase(k,phase)
     double k, phase;
{
    double t, t2, t3, pt, m, mprime, f;
    int apcor = FALSE;

    k += phase; 		      /* Add phase to new moon time */
    t = k / 1236.85;		      /* Time in Julian centuries from
					 1900 January 0.5 */
    t2 = t * t; 		      /* Square for frequent use */
    t3 = t2 * t;		      /* Cube for frequent use */
    pt = 2415020.75933		      /* Mean time of phase */
	 + synmonth * k
	 + 0.0001178 * t2
	 - 0.000000155 * t3
	 + 0.00033 * dsin(166.56 + 132.87 * t - 0.009173 * t2);

    m = 359.2242                      /* Sun's mean anomaly */
	+ 29.10535608 * k
	- 0.0000333 * t2
	- 0.00000347 * t3;
    mprime = 306.0253                 /* Moon's mean anomaly */
	+ 385.81691806 * k
	+ 0.0107306 * t2
	+ 0.00001236 * t3;
    f = 21.2964                       /* Moon's argument of latitude */
	+ 390.67050646 * k
	- 0.0016528 * t2
	- 0.00000239 * t3;
    if((phase < 0.01) || (abs(phase - 0.5) < 0.01)){

       /* Corrections for New and Full Moon */

       pt +=	 (0.1734 - 0.000393 * t) * dsin(m)
		+ 0.0021 * dsin(2 * m)
		- 0.4068 * dsin(mprime)
		+ 0.0161 * dsin(2 * mprime)
		- 0.0004 * dsin(3 * mprime)
		+ 0.0104 * dsin(2 * f)
		- 0.0051 * dsin(m + mprime)
		- 0.0074 * dsin(m - mprime)
		+ 0.0004 * dsin(2 * f + m)
		- 0.0004 * dsin(2 * f - m)
		- 0.0006 * dsin(2 * f + mprime)
		+ 0.0010 * dsin(2 * f - mprime)
		+ 0.0005 * dsin(m + 2 * mprime);
       apcor = TRUE;
    }else if((abs(phase - 0.25) < 0.01 || (abs(phase - 0.75) < 0.01))){
      pt +=	 (0.1721 - 0.0004 * t) * dsin(m)
	        + 0.0021 * dsin(2 * m)
		- 0.6280 * dsin(mprime)
		+ 0.0089 * dsin(2 * mprime)
		- 0.0004 * dsin(3 * mprime)
		+ 0.0079 * dsin(2 * f)
		- 0.0119 * dsin(m + mprime)
		- 0.0047 * dsin(m - mprime)
		+ 0.0003 * dsin(2 * f + m)
		- 0.0004 * dsin(2 * f - m)
		- 0.0006 * dsin(2 * f + mprime)
		+ 0.0021 * dsin(2 * f - mprime)
		+ 0.0003 * dsin(m + 2 * mprime)
		+ 0.0004 * dsin(m - 2 * mprime)
		- 0.0003 * dsin(2 * m + mprime);
       if(phase < 0.5)
	 /* First quarter correction */
	 pt += 0.0028 - 0.0004 * dcos(m) + 0.0003 * dcos(mprime);
       else
	 /* Last quarter correction */
	 pt += -0.0028 + 0.0004 * dcos(m) - 0.0003 * dcos(mprime);
       apcor = TRUE;
    }
    if (!apcor) {
      fprintf(stderr, "TRUEPHASE called with invalid phase selector.\n");
      abort();
    }
    return pt;
}


/*   PHASEHUNT	--  Find time of phases of the moon which surround the
		    current date.  Five phases are found, starting and
		    ending with the new moons which bound the  current
		    lunation.  */

void phasehunt(sdate,phases)
     double sdate, phases[5];
{
  double adate, k1, k2, nt1, nt2;
  int yy, mm, dd;
  
  adate = sdate - 45;
  
  inv_julian(adate, &mm, &dd, &yy);
  k1 = floor((yy + ((mm - 1) * (1.0 / 12.0)) - 1900) * 12.3685);
  
  adate = nt1 = meanphase(adate, k1);
  while(TRUE){
    adate += synmonth;
    k2 = k1 + 1;
    nt2 = meanphase(adate, k2);
    if (nt1 <= sdate && nt2 > sdate)
      break;
    nt1 = nt2;
    k1 = k2;
  }
  phases[0] = truephase(k1, 0.0);
  phases[1] = truephase(k1, 0.25);
  phases[2] = truephase(k1, 0.5);
  phases[3] = truephase(k1, 0.75);
  phases[4] = truephase(k2, 0.0);
}


/*  KEPLER  --	 Solve the equation of Kepler.	*/
/* 
  double kepler(m,ecc)
  double m, ecc;
  {
  double e, delta;
  
  #define EPSILON 1E-6         
  
  e = m = torad(m);
  do{
  delta = e - ecc * sin(e) - m;
  e -= delta / (1 - ecc * cos(e));
  }while(abs(delta) > EPSILON);
  return e;
  }
*/

/*  PHASE  --  Calculate phase of moon as a fraction:
  
    The  argument  is  the  time  for  which  the  phase is requested,
    expressed as a Julian date and fraction.  Returns  the  terminator
    phase  angle  as a percentage of a full circle (i.e., 0 to 1), and
    stores into pointer arguments  the	illuminated  fraction  of  the
    Moon's  disc, the Moon's age in days and fraction, the distance of
    the Moon from the centre of the Earth, and	the  angular  diameter
    subtended  by the Moon as seen by an observer at the centre of the
    Earth.
*/

double phase(pdate, pphase, mage, dist, angdia, sudist, suangdia)
     double  pdate;
     double  *pphase;		      /* Illuminated fraction */
     double  *mage;		      /* Age of moon in days */
     double  *dist;		      /* Distance in kilometres */
     double  *angdia;		      /* Angular diameter in degrees */
     double  *sudist;		      /* Distance to Sun */
     double  *suangdia;               /* Sun's angular diameter */
{

  double Day, N, M, Ec, Lambdasun, ml, MM, MN, Ev, Ae, A3, MmP,
    mEc, A4, lP, V, lPP, NP, y, x, Lambdamoon, BetaM,
    MoonAge, MoonPhase,
    MoonDist, MoonDFrac, MoonAng, MoonPar,
    F, SunDist, SunAng;
  
  /* Calculation of the Sun's position */
  Day = pdate - epoch;		           /* Date within epoch */
  N = fixangle((360 / 365.2422) * Day);    /* Mean anomaly of the Sun */
  M = fixangle(N + elonge - elongp);	   /* Convert from perigee
					      co-ordinates to epoch 1980.0 */
  Ec = eccentric_anomaly(torad(M),eccent); /* Solve equation of Kepler */
  Ec = sqrt((1 + eccent) / (1 - eccent)) * tan(Ec / 2);
  Ec = 2 * todeg(atan(Ec));		   /* True anomaly */
  Lambdasun = fixangle(Ec + elongp);       /* Sun's geocentric ecliptic
					      longitude */
    /* Orbital distance factor */
  F = ((1 + eccent * cos(torad(Ec))) / (1 - eccent * eccent));
  SunDist = sunsmax / F;		   /* Distance to Sun in km */
  SunAng = F * sunangsiz;                  /* Sun's angular size in degrees */
  
  
  /* Calculation of the Moon's position */
  /* Moon's mean longitude */
  ml = fixangle(13.1763966 * Day + mmlong);
  
  /* Moon's mean anomaly */
  MM = fixangle(ml - 0.1114041 * Day - mmlongp);
  
  /* Moon's ascending node mean longitude */
  MN = fixangle(mlnode - 0.0529539 * Day);
  
  /* Evection */
  Ev = 1.2739 * sin(torad(2 * (ml - Lambdasun) - MM));
  
  /* Annual equation */
  Ae = 0.1858 * sin(torad(M));
  
  /* Correction term */
  A3 = 0.37 * sin(torad(M));
  
  /* Corrected anomaly */
  MmP = MM + Ev - Ae - A3;
  
  /* Correction for the equation of the centre */
  mEc = 6.2886 * sin(torad(MmP));
  
  /* Another correction term */
  A4 = 0.214 * sin(torad(2 * MmP));
  
  /* Corrected longitude */
  lP = ml + Ev + mEc - Ae + A4;
  
  /* Variation */
  V = 0.6583 * sin(torad(2 * (lP - Lambdasun)));
  
  /* True longitude */
  lPP = lP + V;
  
  /* Corrected longitude of the node */
  NP = MN - 0.16 * sin(torad(M));
  
  /* Y inclination coordinate */
  y = sin(torad(lPP - NP)) * cos(torad(minc));
  
  /* X inclination coordinate */
  x = cos(torad(lPP - NP));
  
  /* Ecliptic longitude */
  Lambdamoon = todeg(atan2(y, x));
  Lambdamoon += NP;
  
  /* Ecliptic latitude */
  BetaM = todeg(asin(sin(torad(lPP - NP)) * sin(torad(minc))));
  
  /* Calculation of the phase of the Moon */
  /* Age of the Moon in degrees */
  MoonAge = lPP - Lambdasun;
  
  /* Phase of the Moon */
  MoonPhase = (1 - cos(torad(MoonAge))) / 2;
  
  /* Calculate distance of moon from the centre of the Earth */
  MoonDist = (msmax * (1 - mecc * mecc)) / (1 + mecc * cos(torad(MmP + mEc)));
  
  /* Calculate Moon's angular diameter */
  MoonDFrac = MoonDist / msmax;
  MoonAng = mangsiz / MoonDFrac;
  
  /* Calculate Moon's parallax */
  MoonPar = mparallax / MoonDFrac;
  
  *pphase = MoonPhase;
  *mage = synmonth * (fixangle(MoonAge) / 360.0);
  *dist = MoonDist;
  *angdia = MoonAng;
  *sudist = SunDist;
  *suangdia = SunAng;
  return fixangle(MoonAge) / 360.0;
}

/*-------------------------------------------------------------------------*/
/* Function `moon':                                                        */
/* Moon phases for kal                                                     */
/*-------------------------------------------------------------------------*/
void moon(jd,color,out)
     double jd;
     int color; 
     FILE *out;
{
  int ph, lunation;
  int i, yy, mm, dd, hh, mmm, ss;
  int aom_d, aom_h, aom_m;
  double p, aom, cphase, cdist, cangdia, csund, csuang;
  double c = 0.0, d;
  double phasar[5];
  char *loc = (char *) calloc(256,sizeof(char));
  char *zname = (char *) calloc(8,sizeof(char));
  char *offset = (char *) calloc(8,sizeof(char));
  
  fprintf(out,"\nSite:\t\t");
  if((d = localmoon(jd,loc,zname,offset)) != -1.0 && d != -2.0)
    fprintf(out,"%s%s%s  (%s)\n\n", MODE_R0(color),loc,MODE_N(color),offset);
  else 
    if(d == -2.0)
      fprintf(out,"(Unknown site: %s)\n\n",getenv("KAL_SITE"));
    else
      fprintf(out,"(No sites file)\n\n");
      
  /* Local time, universal time, and Julian date */
  if(d != -1.0 && d != -2.0){
    inv_julian(d, &mm, &dd, &yy);
    jhms(d, &hh, &mmm, &ss);
    fprintf(out,"%s\t%s%2i:%02i:%02i  %s %2i %i  %s%s\n\n",labels[2],
	    MODE_B1(color),hh,mmm,ss,moname[mm -1],dd,yy,zname,MODE_N(color));
  }

  inv_julian(jd, &mm, &dd, &yy);
  jhms(jd, &hh, &mmm, &ss);
  fprintf(out,"%s\t%s%2i:%02i:%02i  %s %2i %i  %s%s\n\n",labels[1],
	  MODE_B1(color),hh,mmm,ss,moname[mm -1],dd,yy,UTIME,MODE_N(color));
  
  
  fprintf(out,"%s\t%.5f\n\n", labels[0], jd);


  /* Information about the Moon */
  p = phase(jd, &cphase, &aom, &cdist, &cangdia, &csund, &csuang);
  
  /* Age of moon */
  /* 
   * The original aom was a little confusing.
   * Hence let's try something different:
  */
  phasehunt(jd, phasar);
  while(phasar[0] > jd){
    c += 1.0;
    phasehunt(jd - c, phasar);
  }
  while(phasar[4] < jd){
    c -= 1.0;
    phasehunt(jd - c, phasar);
  }
  aom = jd - phasar[0]; 
  
  aom_d = (int) aom;
  aom_h = (int) (24 * (aom - floor(aom)));
  aom_m = (int) (1440 * (aom - floor(aom))) % 60;
  
  fprintf(out,"%s\t%i day%s, %i hour%s, %i minute%s\n\n", labels[4],
	  EPL(aom_d), EPL(aom_h), EPL(aom_m));

  /* Moon phase */
  ph = (int) rint(cphase * 100);
  fprintf(out,"%s\t%d%% ",labels[5], ph);
  switch(ph){
  case 0:
    fprintf(out,"(%s)\n\n", phasname[0]);
    break;
  case 100:
    fprintf(out,"(%s)\n\n", phasname[2]);
    break;
  default:
    if(jd < phasar[2]) 
      fprintf(out,"(%s)\n\n", phasname[1]);
    else 
      fprintf(out,"(%s)\n\n", phasname[3]);
  }
  
  
  /* Moon phase */
  /*
    sprintf(tbuf, "%d%%   (0%% = New, 100%% = Full)",
    (int) (cphase * 100));
    while (strlen(tbuf) < 40) {
    strcat(tbuf, " ");
    }
  */

  /* Moon distance */
  /*
    sprintf(tbuf, "%ld kilometres, %.1f Earth radii.",
    (long) cdist, cdist / earthrad);
  */
  
  /* Moon subtends */
  /*
    sprintf(tbuf, "%.4f degrees.", cangdia);
  */

  /* Information about the Sun */
  /* Sun's distance */
  /*
    sprintf(tbuf, "%.0f kilometres, %.3f astronomical units.",
    csund, csund / sunsmax);
  */

  /* Sun subtends */
  /*
    sprintf(tbuf, "%.4f degrees.", csuang);
  */
  
  if(jd > nptime){
    lunation = floor(((phasar[0] + 7) - lunatbase) / synmonth) + 1;
    for(i = 0; i < 5; i++){
      fmt_phase_time(phasar[i], i, lunation, loc, zname, offset, out);
    }
  }
  fprintf(out,"\n");
}
