/*
    This file is part of kal.
    Copyright (C) 1999  Frank W. Josellis

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* riset.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "kal.h"
#include "moon.h"


static char *site0 = "Greenwich";
extern char *prog;

#define UNIX_EPOCH  2440587.5   /* Julian date of Jan 01 1970 0:00:00 UTC  */
#define EPOCH       2451543.5   /* Julian date of Dec 31 1999 0:00:00 UTC  */
#define TMAX     2147483647.0   /* 2^31 - 1 */
#define TMIN    -2147483648.0 

#define deg2rad(x) ((x) * PI/180.0)
#define rad2deg(x) ((x) * 180.0/PI)
#define fix1(x) ((x) - 360.0 * (floor((x)/360.0)))    /* Fundamental angle */
#define fix2(x) (fix1((x)) > 180.0) ? fix1((x)) - 360.0 : fix1((x))

#define sind(x) sin(deg2rad((x)))
#define cosd(x) cos(deg2rad((x)))
#define tand(x) tan(deg2rad((x)))


/* 
 * The sun's altitude at the moment of rise and set.
 * Upper limb touches horizon, assuming standard atmospheric refraction.
 */   
const double h_sun = -0.833;


/* 
 * The moon's altitude at the moment of rise and set.
 * Center touches horizon, assuming standard atmospheric refraction.
 */   
const double h_moon = -0.583;


/*-------------------------------------------------------------------------*/
/* Function `jd2unix'                                                      */
/* Convert Julian date to Unix time                                        */
/*-------------------------------------------------------------------------*/
time_t jd2unix(jd)
     double jd;
{
  jd = (jd - UNIX_EPOCH) * 86400.0;
  if(jd > TMAX || jd < TMIN)
    return((time_t) TMIN);
  else 
    return((time_t) jd);
}
    

/*-------------------------------------------------------------------------*/
/* Function `unix2jd'                                                      */
/* Convert Unix time to Julian date                                        */
/*-------------------------------------------------------------------------*/
double unix2jd(t)
     time_t t;
{
  return(((double) t)/86400.0 + UNIX_EPOCH); 
}


/*-------------------------------------------------------------------------*/
/* Function `tz_offset'                                                    */
/* Time zone offset                                                        */
/*-------------------------------------------------------------------------*/
double tz_offset(jd,zone,offset)
     double jd;
     char *zone, *offset;
{
  double diff, hh, mm;
  char pm, *tz; 
  time_t lt;
  struct tm *LT, *UT;
  
  tz = (char *) calloc(strlen(zone) + 4,sizeof(char));
  sprintf(tz,"TZ=%s",zone);
  putenv(tz);
  tzset();

  lt = jd2unix(jd);

  /* 
   *  Begin compromise between Linux and BSD 
   */
  UT  = localtime(&lt);
  LT = (struct tm *) malloc(sizeof(struct tm));
  *LT = *UT;
  UT = gmtime(&lt);

  /* Offset in sec */
  diff = 3600 * LT->tm_hour + 60 * LT->tm_min + LT->tm_sec 
    - (3600 * UT->tm_hour + 60 * UT->tm_min + UT->tm_sec);
  diff += (LT->tm_year == UT->tm_year) ? 
    (LT->tm_yday - UT->tm_yday) * 86400.0 :
    (LT->tm_year - UT->tm_year) * 86400.0;
  /* 
   *  End compromise 
   */

  pm = (diff < 0.0) ? '-' : '+';
  hh = fabs(diff)/3600;
  mm = fmod(hh,1.0) * 60;
  /*ss = fmod(mm,1.0) * 60;*/
  sprintf(offset,"%c%02i%02i", pm, (int) hh, (int) mm);

  /* Offset in days */
  return(diff/86400);
}


/*-------------------------------------------------------------------------*/
/* Function `eccentric_anomaly'                                            */
/* Solve Kepler's equation with Newton's method                            */
/*-------------------------------------------------------------------------*/
double eccentric_anomaly(M,e)
     double M, e;
{
  double E = M, delta;
  
  /* M is given in radian */
  do{
    delta = E - e * sin(E) - M ;
    E -= delta/(1 - e * cos(E));
  }while(abs(delta) >= 1E-9);
 
  return(E);
}


/*-------------------------------------------------------------------------*/
/* Function `positionSun'                                                  */
/*-------------------------------------------------------------------------*/
void positionSun(jd,lambda,beta,r)
     double jd, *lambda, *beta, *r;
{
  double d, w, e, M, E, x, y, v;

  d = jd - EPOCH;

  /* Orbital elements of the Sun */
  w = 282.9404 + 4.70935E-5 * d;       /* Argument of perihelion */
  e = 0.016709 - 1.151E-9 * d;         /* Numeric eccentricity   */
  M = 356.0470 + 0.9856002585 * d;     /* Mean anomaly           */
  
  /* Eccentric anomaly (in radian).
   * The eccentric anomaly E is related to the mean anomaly M
   * by Kepler's equation
   *   
   *   E = M + e sin(E)
   *
   * Thus one may use Newton's method to determine E.
   */
  E = eccentric_anomaly(deg2rad(M),e);

  
  /* Get the true anomaly v and the distance r from eccentricity e and
   * eccentric anomaly E. The relation is
   *
   *   r cos(v) = cos(E) - e
   *   r sin(v) = sqrt(1 - e*e) sin(E)
   *
   */
  x = cos(E) - e;
  y = sin(E) * sqrt(1.0 - e*e);
  
  v  = rad2deg(atan2(y,x));
  *r = sqrt(x * x + y * y); 
 
  /* The sun's longitude in degrees */
  *lambda = fix1(v + w);  
  *beta = 0.0;
}


/*-------------------------------------------------------------------------*/
/* Function `positionMoon'                                                 */
/*-------------------------------------------------------------------------*/
void positionMoon(jd,lambda,beta,r)
     double jd, *lambda, *beta, *r;
{
  double d, N, i, w, a, e, M, E, v, x, y, z;
  
  d = jd - EPOCH;

  /* Orbital elements of the Moon */
  N = 125.1228 - 0.0529538083 * d;        /* Longitude of ascending node   */
  i = 5.1454;                             /* Inclination to the ecliptic   */
  w = 318.0634 + 0.1643573223 * d;        /* Argument of perigee           */
  a = 60.2666;                            /* Semi-major axis (Earth radii) */
  e = 0.054900;                           /* Numeric Eccentricity          */
  M = 115.3654 + 13.0649929509 * d;       /* Mean anomaly                  */
  
  /* Eccentric anomaly (in radian) */
  E = eccentric_anomaly(deg2rad(M),e);
  
  /* Get the true anomaly v and the distance r from eccentricity e and
   * eccentric anomaly E. The relation is
   *
   *   r cos(v) = a (cos(E) - e)
   *   r sin(v) = a (sqrt(1 - e*e) sin(E))
   *
   */
  x = a * (cos(E) - e);
  y = a * (sin(E) * sqrt(1.0 - e*e));
  
  v = rad2deg(atan2(y,x));
  *r = sqrt(x * x + y * y);
 
  /* Get the position in space */
  x = *r * (cosd(N) * cosd(v + w) - sind(N) * sind(v + w) * cosd(i));
  y = *r * (sind(N) * cosd(v + w) + cosd(N) * sind(v + w) * cosd(i));
  z = *r * (sind(v + w) * sind(i));
  
  /* The moon's angular coordinates (in degrees) */
  *lambda = fix1(rad2deg(atan2(y,x)));  
  *beta   = rad2deg(atan2(z,sqrt(x * x + y * y)));

}


/*-------------------------------------------------------------------------*/
/* Function `perturbMoon'                                                  */
/*-------------------------------------------------------------------------*/
void perturbMoon(jd,lambda,beta,r)
     double jd, *lambda, *beta, *r;
{
  double d, Ms, Mm, Nm, ws, wm, Ls, Lm, D, F; 
  
  d = jd - EPOCH;

  /* Mean anomaly of the sun and the moon */ 
  Ms = 356.0470 + 0.9856002585 * d;
  Mm = 115.3654 + 13.0649929509 * d;

  /* Longitude of the moon's ascending node   */
  Nm = 125.1228 - 0.0529538083 * d;

  /* Argument of perihelion for the sun */
  ws = 282.9404 + 4.70935E-5 * d;

  /* Argument of perigee for the moon */
  wm = 318.0634 + 0.1643573223 * d;
  
  /* Mean longitude of the sun and the moon */
  Ls = Ms + ws;
  Lm = Mm + wm + Nm;

  /* Mean elongation of the moon */
  D = Lm - Ls;
  
  /* Argument of latitude for the moon */
  F = Lm - Nm;

  /* Perturbation of ecliptic longitude */
  *lambda += -1.274 * sind(Mm - 2*D) 
    + 0.658 * sind(2*D)             
    - 0.186 * sind(Ms)              
    - 0.059 * sind(2*Mm - 2*D)
    - 0.057 * sind(Mm - 2*D + Ms)
    + 0.053 * sind(Mm + 2*D)
    + 0.046 * sind(2*D - Ms)
    + 0.041 * sind(Mm - Ms)
    - 0.035 * sind(D)         
    - 0.031 * sind(Mm + Ms)
    - 0.015 * sind(2*F - 2*D)
    + 0.011 * sind(Mm - 4*D);
  
  /* Perturbation of ecliptic latitude */
  *beta += -0.173 * sind(F - 2*D)
    - 0.055 * sind(Mm - F - 2*D)
    - 0.046 * sind(Mm + F - 2*D)
    + 0.033 * sind(F + 2*D)
    + 0.017 * sind(2*Mm + F);

  /* Perturbation of radius */
  *r += -0.58 * cosd(Mm - 2*D) - 0.46 * cosd(2*D);

  *lambda = fix1(*lambda); 
}
  
/*-------------------------------------------------------------------------*/
/* Function `ecl2equat'                                                    */
/* Right ascension and declination from ecliptic longitude and latitude    */
/*-------------------------------------------------------------------------*/
void ecl2equat(jd,lambda,beta,alpha,delta)
     double jd, lambda, beta, *alpha, *delta;
{
  double ecl;

  /* Obliquity of the ecliptic */
  ecl = 23.4393 - 3.563E-7 * (jd - EPOCH);

  /* Get right ascension alpha and declination delta */
  *delta = 
    asin(sind(ecl) * sind(lambda) * cosd(beta) + cosd(ecl) * sind(beta));

  *alpha = 
    atan2(cosd(ecl) * sind(lambda) * cosd(beta) - sind(ecl) * sind(beta),
	  cosd(beta) * cosd(lambda));

  /* Express angles in degrees */
  *alpha = fix1(rad2deg(*alpha));
  *delta = rad2deg(*delta);
}


/*-------------------------------------------------------------------------*/
/* Function `topocentric'                                                  */
/* Topocentric RA and Dec from geocentric                                  */ 
/*-------------------------------------------------------------------------*/
void topocentric(ha,r,alpha,delta,gclat,rho,topdelta,obj)
     double ha, r, alpha, delta, gclat, rho, *topdelta; 
     short obj;
{
  double par, g;

  if(obj)
    par = rad2deg(asin(1/r));
  else
    par = (8.794/3600)/r;
    
  g = rad2deg(atan2(tand(gclat),cosd(ha)));
  *topdelta = delta - par * rho * sind(gclat) * sind(g - delta)/sind(g);
}


/*-------------------------------------------------------------------------*/
/* Function `riset'                                                        */
/* JD of rise or set                                                       */ 
/*-------------------------------------------------------------------------*/
double riset(st,tt,alpha,topdelta,gclat,rho,sign,obj)
     double st, tt, alpha, topdelta, gclat, rho;
     short sign, obj;
{
  double h, ha, T, alt, t, lambda, beta, radius, delta;

  if(obj)
    h = h_moon;
  else
    h = h_sun;

  do{ 
    ha = fix2(st - alpha);
    st = fix1(st - ha);
    tt -= ha/360.985647352;

    T = 
      (sind(h) - sind(gclat) * sind(topdelta))/(cosd(gclat) * cosd(topdelta));
    
    if(fabs(T) <= 1.0){
      t = tt - sign * rad2deg(acos(T))/360.985647352;
      if(obj){
	positionMoon(t,&lambda,&beta,&radius);
	perturbMoon(t,&lambda,&beta,&radius);
      }
      else 
	positionSun(t,&lambda,&beta,&radius);

      ecl2equat(t,lambda,beta,&alpha,&delta);
      topocentric(ha,radius,alpha,delta,gclat,rho,&topdelta,obj);
      alt = sind(gclat) * sind(topdelta) + cosd(gclat) * cosd(topdelta) * T;
      alt = rad2deg(asin(alt));
    } 
    else 
      return(0.0);
  }while(fabs(h - alt) >= 1E-8);
  
  return(t);
}

/*-------------------------------------------------------------------------*/
/* Function `shine'                                                        */
/*-------------------------------------------------------------------------*/
void shine(jd,lon,lat,off,obj,out)
     double jd, lon, lat, off;
     short obj; 
     FILE *out;
{
  int i;
  double R = 0.0, S = 0.0; 
  double d, st, gst, ha, T,
    alpha, delta, lambda, beta, radius, tt, gclat, rho, topdelta, alt = 0.0,
    noon[3], rise[3], set[3];
  time_t rise_t, set_t;
  struct tm *LT;
  char risetime[42], settime[42], tbuf[32];
  
  d = jd - EPOCH;

  gclat = lat - 0.1924 * sind(2 * lat);
  rho = 0.99883 + 0.00167 * cosd(2 * lat);

  /* Default output */
  sprintf(risetime,"\t\tRise: None\n");
  sprintf(settime,"\t\tSet:  None\n");

  for(i = 0; i < 3; i++){
    /* 
     * Sidereal Time of Greenwich at 12:00:00 UTC in degrees.
     * Note: This is actually the sun's mean longitude  L = M + w at time d. 
     */ 
    gst = 278.9874 + 0.985647352 * (d + i - 1);
  
    /* Local Sidereal Time at local noon, using the time zone */
    st = fix1(gst - off * 360.985647352 + lon);
    
    /* Universal Time corresponding to local noon */
    noon[i] = EPOCH + floor(d + i - 1) + 0.5 - off;
  
    /* The position at local noon */
    if(obj){
      positionMoon(noon[i],&lambda,&beta,&radius);
      perturbMoon(noon[i],&lambda,&beta,&radius);
    }
    else
      positionSun(noon[i],&lambda,&beta,&radius);

    ecl2equat(noon[i],lambda,beta,&alpha,&delta);

    /* Hour angle at local noon */
    ha = fix2(st - alpha);
    
    /* 
     * Get the transit time and corresponding position, 
     * local sidereal time, and hour angle.
     */ 
    tt = noon[i];
    while(fabs(ha) > 1E-8){
      st = fix1(st - ha); 
      tt -= ha/360.985647352; 
      if(obj){
	positionMoon(tt,&lambda,&beta,&radius);
	perturbMoon(tt,&lambda,&beta,&radius);
      }
      else
	positionSun(tt,&lambda,&beta,&radius);
      
      ecl2equat(tt,lambda,beta,&alpha,&delta);
      ha = fix2(st - alpha);
    }
    topocentric(ha,radius,alpha,delta,gclat,rho,&topdelta,obj);

    /* Altitude at transit time */
    if(i == 1){
      alt = sind(gclat) * sind(topdelta) 
	+ cosd(gclat) * cosd(topdelta) * cosd(ha);
      alt = rad2deg(asin(alt));
    }
      
    /* Now get the time of rise and set */
    rise[i] = riset(st,tt,alpha,topdelta,gclat,rho,+1,obj);
    set[i]  = riset(st,tt,alpha,topdelta,gclat,rho,-1,obj);
  }
  
  for(i = 0; i < 3; i++){
    if(fabs(rise[i] - noon[1]) < 0.5){
      if(fabs(rise[i] - UNIX_EPOCH) * 86400 > TMAX){
	T = 24.0 * fmod(rise[i] - noon[1] + 0.5,1.0);
	sprintf(risetime,"\t\tRise: %2i:%02i GMT%+.0f\n", 
		(int) T, (int) ((T - floor(T)) * 60), 24 * off);
      }
      else {
	rise_t = jd2unix(rise[i]);
	LT = localtime(&rise_t);
	strftime(tbuf,32,"%H:%M %Z",LT);
	sprintf(risetime,"\t\tRise: %s\n",tbuf);
      }
      R = rise[i];
    }

    if(fabs(set[i] - noon[1]) < 0.5){
      if(fabs(set[i] - UNIX_EPOCH) * 86400 > TMAX){
	T = 24.0 * fmod(set[i] - noon[1] + 0.5,1.0);
	sprintf(settime,"\t\tSet:  %2i:%02i GMT%+.0f\n", 
		(int) T, (int) ((T - floor(T)) * 60), 24 * off);
      }
      else {
	set_t = jd2unix(set[i]);
	LT = localtime(&set_t);
	strftime(tbuf,32,"%H:%M %Z",LT);
	sprintf(settime,"\t\tSet:  %s\n",tbuf);
	S = set[i];
      }
      S = set[i];
    }
  }

  if(obj)
    fprintf(out,"\t\tMoon\n\t\t----\n");
  else
    fprintf(out,"\t\tSun\n\t\t---\n");

  if(R == 0.0 && S == 0.0){
    if((obj && alt < h_moon) || (!obj && alt < h_sun))
      fprintf(out,"\t\tNever up\n\n");
    else
      fprintf(out,"\t\tAlways up\n\n");
  } else 
    if(R < S)
      fprintf(out,"%s%s\n",risetime,settime);
    else
      fprintf(out,"%s%s\n",settime,risetime);
}


/*-------------------------------------------------------------------------*/
/* Function `get_local'                                                    */
/*-------------------------------------------------------------------------*/
int get_local(buf,mode)
     char buf[], *mode;
{ 
  int match = -1;
  char *sitename = getenv("KAL_SITE"), *sites_user, *home = getenv("HOME");
  FILE *sitefile_user, *sitefile_sys;
  
  if(home){
    sites_user = (char *) calloc(strlen(home) + 12,sizeof(char));
    sprintf(sites_user,"%s/.kal-sites",home);
  }
  
  sitefile_user = (home) ? fopen(sites_user,"r") : NULL;
  sitefile_sys  = fopen(SITES_SYS,"r");
  
  if(!sitefile_user && !sitefile_sys){
    if(mode)
      fprintf(stderr,"%s: No sites file\n",mode);
    return(-1);
  }

  /* Get site name or use default */
  if(sitename && strlen(sitename) > 0)
    while(strchr(sitename,95)) *strchr(sitename,95) = 32;
  else 
    sitename = site0;

  if(sitefile_user){
    do{
      fgets(buf,1024,sitefile_user);
      match = strncmp(buf,sitename,strlen(sitename));
    }while(!feof(sitefile_user) && match);
    fclose(sitefile_user);
  }

  if(sitefile_sys){
    if(match)
      do{
	fgets(buf,1024,sitefile_sys);
	match = strncmp(buf,sitename,strlen(sitename));
      }while(!feof(sitefile_sys) && match);
    fclose(sitefile_sys);
  }

  if(match){
    if(mode)
      fprintf(stderr,"%s: Unknown site: %s\n", prog, sitename);
    return(-2);
  }

  return(0);
}


/*-------------------------------------------------------------------------*/
/* Function `updown'                                                       */
/*-------------------------------------------------------------------------*/
void updown(jd,color,out)
     double jd;
     int color;
     FILE *out;
{
  int sign;
  char *sitename, *zone, *stdoff, *latitude, *longitude, buf[1024];
  char *offset, pm;
  double lon, lat, off;

  if(get_local(buf,prog)) return;
  
  sitename  = strtok(buf,";");
  latitude  = strtok(NULL," ");
  longitude = strtok(NULL," ");
  zone      = strtok(NULL," ");
  stdoff    = strtok(NULL," ");

  /* Avoid possible problems with trailing '\n' */
  if(!stdoff && strchr(zone,10)) *strchr(zone,10) = 0;

  sign = (strchr(latitude,'S')) ? -1.0 : 1.0;
  lat = sign * atof(strtok(latitude,":"));
  lat += sign * atof(strtok(NULL,":"))/60.0;
  lat += sign * atof(strtok(NULL,":"))/3600.0;

  sign = (strchr(longitude,'W')) ? -1.0 : 1.0;
  lon = sign * atof(strtok(longitude,":"));
  lon += sign * atof(strtok(NULL,":"))/60.0;
  lon += sign * atof(strtok(NULL,":"))/3600.0;
  
  jd = floor(jd + 0.5);
  offset = (char *) calloc(8,sizeof(char));
  if(fabs(jd - UNIX_EPOCH) * 86400 > TMAX){
    if(stdoff){
      pm = (strchr(stdoff,'-')) ? '-' : '+';
      sprintf(offset,"%c%02i00",pm,abs(atoi(stdoff)));
    }
    else 
      sprintf(offset,"+0000");
    off = atof(offset)/2400;
  }
  else 
    off = tz_offset(jd,zone,offset);
  
  jd -= off;
  fprintf(out,"\nRise and set times for %s%s%s\n\n",
	  MODE_R0(color),sitename,MODE_N(color)); 
  shine(jd,lon,lat,off,0,out); /* sun  */ 
  shine(jd,lon,lat,off,3,out); /* moon */
  fprintf(out,"Local noon is %.5f  TZ offset is %s\n\n",jd,offset);
}


/*-------------------------------------------------------------------------*/
/* Function `localmoon'                                                    */
/* Local time for moon                                                     */
/*-------------------------------------------------------------------------*/
double localmoon(jd,loc,zname,offset)
     double jd;
     char *loc, *zname, *offset;
{
  double off;
  char buf[1024], *sitename, pm, *zone, *stdoff;
  time_t lt;
  struct tm *LT;

  if((off = (double) get_local(buf,NULL))) 
    return(off);

  sitename = strtok(buf,";");
  sprintf(loc,"%s",sitename);
  strtok(NULL," ");
  strtok(NULL," ");
  zone = strtok(NULL," ");
  stdoff    = strtok(NULL," ");
  if(!stdoff && strchr(zone,10)) *strchr(zone,10) = 0;
  
  if(fabs(jd - UNIX_EPOCH) * 86400 > TMAX){
    if(stdoff){
      pm = (strchr(stdoff,'-')) ? '-' : '+';
      sprintf(offset,"%c%02i00",pm,abs(atoi(stdoff)));
    }
    else 
      sprintf(offset,"+0000");
    off = atof(offset)/2400;
  }
  else 
    off = tz_offset(jd,zone,offset);

  if(fabs(jd + off - UNIX_EPOCH) * 86400 > TMAX)
    sprintf(zname,"GMT%+i",atoi(offset)/100);
  else {
    lt = jd2unix(jd + off);
    LT = localtime(&lt);
    strftime(zname,8,"%Z",LT);
  }

  return(jd + off);
}
